# -*- coding: utf-8 -*-
# Copyright (c) 2003-2006 ETH Zurich, D-MATL. Written by Lorenz Textor. All rights reserved.
# See also LICENSE.txt


from MySQLdb import escape_string
from urllib import urlencode, quote, unquote
from time import localtime, strftime
from re import escape, match
from popen2 import popen2
from tempfile import mkstemp
import os
import smtplib
import MimeWriter
import base64
import StringIO
from zipfile import ZipFile, ZIP_DEFLATED

from Products.SilvaExtETHLayout.helpers import get_language_values
from Products.SilvaExtPersonInfo.helpers import getLDAP_authEntry, searchLDAP_data, get_persid, getMembersOfGroup
from Products.SilvaExtETHLayout.install import smtp_host
from Products.SilvaExtVVZ.helpers import get_vvz_cache
from Products.SilvaExtVVZ.VVZCache import CacheEvent, CacheLearningUnit

#Zope
from AccessControl import ClassSecurityInfo, Permissions, getSecurityManager
from Globals import InitializeClass, package_home
from Globals import DTMLFile
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from App.ImageFile import ImageFile
from Products.Silva.helpers import add_and_edit
from Products.ZSQLMethods.SQL import SQL, SQLConnectionIDs
from Products.PythonScripts.standard import newline_to_br, html_quote

from Acquisition import Implicit
from Globals import Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from OFS.Traversable import Traversable
from OFS.Image import Image


# images array used to fix images link for pdf creation
_images = ['banner.jpg','icon_email.gif','icon_pdf.gif','icon_print.gif','lang_sep.gif','lang_d.gif','lang_e.gif','top.gif',
           'checkmark.gif','cross.gif','plus.gif','minus.gif','up.gif','down.gif','delete.gif','edit.gif','caption.jpg',
           'caption2.jpg','bar.gif','bar_a.gif','bar_b.gif','bar_g.gif','bar_t.gif','eth_logo.gif']
           

class Evaluation(Implicit, Persistent, RoleManager, Item, Traversable):
    """
    Evaluation product class.
    
    Contains all methods for the Evaluation tool.
    """
    
    # __implements__ = IDBEditor
    
    meta_type='Evaluation'
    
    security=ClassSecurityInfo()
    
    manage_options = (
        {'label' : 'Edit', 'action' : 'manage_main'},
        {'label' : 'View', 'action' : ''},
        ) + RoleManager.manage_options + Item.manage_options
    
    manage_main = edit_tab = PageTemplateFile('www/EvaluationEdit',
                                                globals(), 
                                                __name__='edit_tab')

## defaults for new attributes
    # _lecturesEditor = ''


## init method
    def __init__(self, id, title, dbConnectionId):
        self.id = id
        self.title = title
        self._dbConnectionId = dbConnectionId
        self._database = ''
        self._error = ''
        self._footer = ''
        self._contactLink = ''
        self._lecturesEditor = ''
        self._bscCourse = ''
        self._mscCourse = ''
        self._semesterGroups = {'BSc':'','MSc':''}
        self._evaluationGroups = {'BSc':'','MSc':''}
        # tell the persistence machinery that a mutable attribute has changed
        self._p_changed = 1


## manage methods
    # methods starting with manage_ require always management rights therefore no security declaration necessary!
    def manage_edit(self):
        """
        Saves changes made in the edit tab."""
        req = self.REQUEST
        
        if ( getattr(req, 'dbConnectionSave', None) ) :
        # save dbConnectionId
            oldDbConnectionId = self.getDbConnectionId()
            dbConnectionId = req.dbConnectionId
            connect = self.setDbConnectionId(dbConnectionId)
            if oldDbConnectionId != self.getDbConnectionId() :
                self._database = ''
            if not dbConnectionId :
                manage_tabs_message = 'ERROR: Please select a database connection.'
            else :
                if not connect :
                    manage_tabs_message = 'ERROR: Could not find database connection "%s".' % dbConnectionId
                else :
                    manage_tabs_message = 'Database connection changed to "%s".' % dbConnectionId
            return self.edit_tab(manage_tabs_message=manage_tabs_message)

        if ( getattr(req, 'generalSettingsSave', None) ) :
        # save general settings
            manage_tabs_message = 'General settings saved.'
            self._database = req.database
            self._lecturesEditor = req.lecturesEditor
            self._bscCourse = req.bscCourse
            self._mscCourse = req.mscCourse
            self._semesterGroups['BSc'] = req.bscSemester
            self._semesterGroups['MSc'] = req.mscSemester
            self._evaluationGroups['BSc'] = req.bscGroup
            self._evaluationGroups['MSc'] = req.mscGroup
            self._footer = req.footer
            self._contactLink = req.contactLink
            # tell the persistence machinery that a mutable attribute has changed
            self._p_changed = 1
            return self.edit_tab(manage_tabs_message=manage_tabs_message)
        
        if ( getattr(req, 'copyImages', None) ) :
        # copy images to root folder
            try:
                root = self.restrictedTraverse('/')
                root.manage_addFolder('images_evaluation','images for the evaluation tool')
                for image in _images:
                    f = open(os.path.join(package_home(globals()), 'www/images', image), 'rb')
                    self.images_evaluation._setObject(image, Image(image, image, f))
                    f.close()
                manage_tabs_message = "Images copied to root folder."
            except:
                manage_tabs_message = """Couldn't copy images to root folder. Be sure that you have enough privileges and that the id "images_evaluation" isn't already in use."""
            return self.edit_tab(manage_tabs_message=manage_tabs_message)
        
        return self.edit_tab()

## PageTemplate methods
    security.declareProtected(Permissions.access_contents_information, 'mainTemplate') 
    mainTemplate = PageTemplateFile('www/mainTemplate', globals())

    security.declareProtected(Permissions.access_contents_information, 'index_html') 
    security.declareProtected(Permissions.access_contents_information, 'index') 
    index_html = index = PageTemplateFile('www/index', globals())

    # questions_ context
    security.declareProtected(Permissions.access_contents_information, 'questions_index') 
    questions_index = PageTemplateFile('www/questions_index', globals())

    security.declareProtected(Permissions.access_contents_information, 'questions_questionnaire') 
    questions_questionnaire = PageTemplateFile('www/questions_questionnaire', globals())

    security.declareProtected(Permissions.access_contents_information, 'questions_submit') 
    questions_submit = PageTemplateFile('www/questions_submit', globals())
    
    # evaluation_ context
    security.declareProtected(Permissions.access_contents_information, 'evaluation_index') 
    evaluation_index = PageTemplateFile('www/evaluation_index', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'evaluation_view') 
    evaluation_view = PageTemplateFile('www/evaluation_view', globals())

    security.declareProtected(Permissions.access_contents_information, 'evaluation_compare') 
    evaluation_compare = PageTemplateFile('www/evaluation_compare', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'evaluation_compareview') 
    evaluation_compareview = PageTemplateFile('www/evaluation_compareview', globals())

    # abstracts_ context
    security.declareProtected(Permissions.access_contents_information, 'abstracts_index') 
    abstracts_index = PageTemplateFile('www/abstracts_index', globals())

    security.declareProtected(Permissions.access_contents_information, 'abstracts_view') 
    abstracts_view = PageTemplateFile('www/abstracts_view', globals())

    security.declareProtected(Permissions.access_contents_information, 'abstracts_save') 
    abstracts_save = PageTemplateFile('www/abstracts_save', globals())

    # admin_ context
    security.declareProtected(Permissions.access_contents_information, 'admin_index') 
    admin_index = PageTemplateFile('www/admin_index', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_lectures') 
    admin_lectures = PageTemplateFile('www/admin_lectures', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_questions') 
    admin_questions = PageTemplateFile('www/admin_questions', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_editstatus') 
    admin_editstatus = PageTemplateFile('www/admin_editstatus', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_viewstatus') 
    admin_viewstatus = PageTemplateFile('www/admin_viewstatus', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_editquestions') 
    admin_editquestions = PageTemplateFile('www/admin_editquestions', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_editanswerset') 
    admin_editanswerset = PageTemplateFile('www/admin_editanswerset', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_setlectureset') 
    admin_setlectureset = PageTemplateFile('www/admin_setlectureset', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_mailevaluation') 
    admin_mailevaluation = PageTemplateFile('www/admin_mailevaluation', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'admin_downloadevaluation') 
    admin_downloadevaluation = PageTemplateFile('www/admin_downloadevaluation', globals())
    
    # helper templates
    security.declareProtected(Permissions.access_contents_information, 'caption') 
    caption = PageTemplateFile('www/caption', globals())
    
    security.declareProtected(Permissions.access_contents_information, 'mailPage') 
    mailPage = PageTemplateFile('www/mailPage', globals())
    
## DTML methods
    frontend_css = DTMLFile('www/frontend.css', globals())
    print_css = DTMLFile('www/print.css', globals())
    javascripts_js = DTMLFile('www/javascripts.js', globals())

## ImageFile methods
    # favicon_ico = ImageFile('www/favicon.ico', globals())
    banner_jpg = ImageFile('www/images/banner.jpg', globals())
    icon_email_gif = ImageFile('www/images/icon_email.gif', globals())
    icon_pdf_gif = ImageFile('www/images/icon_pdf.gif', globals())
    icon_print_gif = ImageFile('www/images/icon_print.gif', globals())
    lang_sep_gif = ImageFile('www/images/lang_sep.gif', globals())
    lang_d_gif = ImageFile('www/images/lang_d.gif', globals())
    lang_e_gif = ImageFile('www/images/lang_e.gif', globals())
    top_gif = ImageFile('www/images/top.gif', globals())
    checkmark_gif = ImageFile('www/images/checkmark.gif', globals())
    cross_gif = ImageFile('www/images/cross.gif', globals())
    plus_gif = ImageFile('www/images/plus.gif', globals())
    minus_gif = ImageFile('www/images/minus.gif', globals())
    up_gif = ImageFile('www/images/up.gif', globals())
    down_gif = ImageFile('www/images/down.gif', globals())
    delete_gif = ImageFile('www/images/delete.gif', globals())
    edit_gif = ImageFile('www/images/edit.gif', globals())
    caption_jpg = ImageFile('www/images/caption.jpg', globals())
    caption2_jpg = ImageFile('www/images/caption2.jpg', globals())
    bar_gif = ImageFile('www/images/bar.gif', globals())
    bar_a_gif = ImageFile('www/images/bar_a.gif', globals())
    bar_b_gif = ImageFile('www/images/bar_b.gif', globals())
    bar_g_gif = ImageFile('www/images/bar_g.gif', globals())
    bar_t_gif = ImageFile('www/images/bar_t.gif', globals())
    eth_logo_gif = ImageFile('www/images/eth_logo.gif', globals())
    evaluation_usermanual_pdf = ImageFile('www/pdf/evaluation_usermanual.pdf', globals())
    

    # by default zope has a "deny unless allowed" policy, but Item and SimpleItem override this and make non protected methods available

## user info methods
    security.declareProtected(Permissions.access_contents_information, 'getUserInfo') 
    def getUserInfo(self):
        """Returns the info for the logged in user"""
        query = "SELECT u_id, sex, language, department FROM `%s`.`student_info` WHERE u_id=%s" % (self.getDatabase(), self._escape(self.getUser()))
        return self._queryString(query)
        
    security.declareProtected(Permissions.access_contents_information, 'setUserInfo') 
    def setUserInfo(self):
        """Saves the passed user info"""
        request = self.REQUEST
        if request.has_key('sex'):
            sex = request['sex']
            language = request['nativeLanguage']
            department = request['department']
            query = """REPLACE INTO `%s`.`student_info` 
                       (`u_id`,`sex`,`language`,`department`) 
                       VALUES 
                       (%s, %s, %s, %s)""" % (self.getDatabase(), self._escape(self.getUser()), self._escape(sex), self._escape(language), self._escape(department))
            self._queryString(query)

## questions_ methods
    security.declareProtected(Permissions.access_contents_information, 'getLinksGeneral') 
    def getLinksGeneral(self):
        """returns a dictionary with information for the list of general questions (semesters)"""
        language = self.getLanguage()
        semkez = self.getCurrentSemester()[0]
        answers = self.getAnswersForLecture(semkez)
        semester = self.getSemesterWithSemkez(semkez)
        a = answers and True or False
        if language == 'de' :
            if semester and semester[0]['open'] == 'Y' :
                link = self.questions_questionnaire.absolute_url()+'?type=general&lecture='+semkez
                name = 'Allgemeine Fragen ('+self.getCurrentSemester()[1]+')'
            else :
                link = ''
                name = 'Evaluation abgeschlossen'
                a = 1
        else :
            if semester and semester[0]['open'] == 'Y' :
                link = self.questions_questionnaire.absolute_url()+'?type=general&lecture='+semkez
                name = 'General questions ('+self.getCurrentSemester()[1]+')'
            else :
                link = ''
                name = 'Evaluation closed'
                a = 1
        return {'name':name, 'link':link, 'answered':a}

    
    security.declareProtected(Permissions.access_contents_information, 'getLinksLectures')
    def getLinksLectures(self):
        """returns a dictionary with information for the list of lectures
           hides/shows also lectures from the lectures list"""
        hide = getattr(self.REQUEST,'hide',None)
        show = getattr(self.REQUEST,'show',None)
        if hide:
            # hide the given lecture
            query = """REPLACE INTO `%s`.`lectures_hide` (u_id, semkez, l_id) VALUES (%s, %s, %s)""" % (self.getDatabase(), self._escape(self.getUser()), self._escape(self.getCurrentSemester()[0]), self._escape(hide))
            self._queryString(query)
        if show:
            # show the given lecture
            query = """DELETE FROM `%s`.`lectures_hide` WHERE u_id=%s AND semkez=%s AND l_id=%s""" % (self.getDatabase(), self._escape(self.getUser()), self._escape(self.getCurrentSemester()[0]), self._escape(show))
            self._queryString(query)
        # get hidden lectures
        query = """SELECT l_id FROM `%s`.`lectures_hide` WHERE u_id=%s AND semkez=%s""" % (self.getDatabase(), self._escape(self.getUser()), self._escape(self.getCurrentSemester()[0]))
        hidden = [x['l_id'] for x in self._queryString(query)]
        language = self.getLanguage()
        if self.isStudent() and self.lectureInscriptionAvailable():
            lectures = self.getInscriptionsForStudent()
            links = []
            a = 0
            for lecture in lectures:
                if lecture['open'] == 'Y' :
                    answers = self.getAnswersForLecture(lecture['l_id'])
                    a = answers and True or False 
                    link = self.questions_questionnaire.absolute_url()+'?type=lecture&lecture='+str(lecture['l_id'])
                    if language == 'de' :
                        links.append({'code':lecture['code'], 'name':lecture['name'], 'lecturer':lecture['lecturer'], 'id':lecture['l_id'], 'answered':a, 'link':link})
                    else :
                        links.append({'code':lecture['code'], 'name':lecture['nameEN'], 'lecturer':lecture['lecturer'], 'id':lecture['l_id'], 'answered':a, 'link':link})
            if language == 'de' :
                closed = "Evaluation abgeschlossen"
            else :
                closed = "Evaluation closed"
            return {'links':links, 'otherlinks':[], 'hidden':hidden, 'title':'', 'othertitle':'', 'closed':closed}
        else:
            # get lectures
            links = []
            otherlinks = []
            a = 0
            semester = self.getSemesterOfStudent()
            # mainCourse is the course for which getSemesterOfStudent() returns semester
            allCourses = self.isStudent()
            if allCourses :
                mainCourse = 'MSc' in allCourses and 'MSc' or 'BSc'
            else :
                mainCourse = ''
            # get lectures of corresponding semester
            for course in ('BSc','MSc') :
                if course == mainCourse :
                    for lecture in self.getLecturesForSemester(semester, course) :
                        if lecture['open'] == 'Y' :
                            answers = self.getAnswersForLecture(lecture['l_id'])
                            a = answers and True or False 
                            if language == 'de' :
                                link = self.questions_questionnaire.absolute_url()+'?type=lecture&lecture='+str(lecture['l_id'])
                                links.append({'code':lecture['code'], 'name':lecture['name'], 'lecturer':lecture['lecturer'], 'id':lecture['l_id'], 'answered':a, 'link':link})
                            else :
                                link = self.questions_questionnaire.absolute_url()+'?type=lecture&lecture='+str(lecture['l_id'])
                                links.append({'code':lecture['code'], 'name':lecture['nameEN'], 'lecturer':lecture['lecturer'], 'id':lecture['l_id'], 'answered':a, 'link':link})
            # get lectures for other semesters (only for BSc students > 2. semester, MSc students and non students)
            if semester not in [1,2] or 'MSc' in allCourses or not allCourses :
                for course in ('BSc','MSc') :
                    if course in allCourses or not allCourses :
                        if course == mainCourse :
                            lectures = self.getLecturesNotForSemester(course)   # get lectures for other semesters of main course
                        else :
                            lectures = self.getLecturesForSemester(None,course) # get lectures for all semesters of non main course
                        for lecture in lectures :
                            if lecture['open'] == 'Y' :
                                answers = self.getAnswersForLecture(lecture['l_id'])
                                a = answers and True or False 
                                if language == 'de' :
                                    link = self.questions_questionnaire.absolute_url()+'?type=lecture&lecture='+str(lecture['l_id'])
                                    otherlinks.append({'code':lecture['code'], 'name':lecture['name'], 'lecturer':lecture['lecturer'], 'id':lecture['l_id'], 'answered':a, 'link':link})
                                else :
                                    link = self.questions_questionnaire.absolute_url()+'?type=lecture&lecture='+str(lecture['l_id'])
                                    otherlinks.append({'code':lecture['code'], 'name':lecture['nameEN'], 'lecturer':lecture['lecturer'], 'id':lecture['l_id'], 'answered':a, 'link':link})
            #semester = str(semester)
            if semester == 0 :
                if language == 'de' :
                    semester = 'unbekanntes '
                else :
                    semester = 'unknown '
            else :
                semester = '%d. ' % semester
            if language == 'de' :
                title = "Alle Vorlesungen "+semester+"Semester"
                othertitle = "Vorlesungen aus anderen Semestern"
                closed = "Evaluation abgeschlossen"
            else :
                title = "All lectures "+semester+"semester"
                othertitle = "Lectures from other semesters"
                closed = "Evaluation closed"
            return {'links':links, 'otherlinks':otherlinks, 'hidden':hidden, 'title':title, 'othertitle':othertitle, 'closed':closed}

    security.declareProtected(Permissions.access_contents_information, 'getQuestionsHTML')
    def getQuestionsHTML(self):
        """returns the html for the questionnaire"""
        type = self.REQUEST.type
        lecture = self.REQUEST.lecture
        questions = self.getQuestions(type, lecture)
        # HTML templates
        header = """<tr class="rowheading">
                        <td width="10" class="align-right"></td>
                        <td class="align-left"></td>
                        <td width="10" class="align-center">1</td>
                        <td width="10" class="align-center">2</td>
                        <td width="10" class="align-center">3</td>
                        <td width="10" class="align-center">4</td>
                        <td width="10" class="align-center">5</td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center">k.A.</td>
                    </tr>"""
        title = """<tr class="rowheading">
                        <td width="10" class="align-left large" colspan="2"><br />%s</td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                    </tr>"""
        radio5 = """<tr class="highlight">
                        <td width="10" class="align-right"><span>%s</span></td>
                        <td class="align-left">%s</td>
                        <td width="10" class="align-center"><input type="radio" name="%s" value="1" /></td>
                        <td width="10" class="align-center"><input type="radio" name="%s" value="2" /></td>
                        <td width="10" class="align-center"><input type="radio" name="%s" value="3" /></td>
                        <td width="10" class="align-center"><input type="radio" name="%s" value="4" /></td>
                        <td width="10" class="align-center"><input type="radio" name="%s" value="5" /></td>
                        <td width="10" class="align-center">&nbsp;</td>                             
                        <td width="10" class="align-center"><input type="radio" name="%s" value="0" /></td>
                    </tr>"""
        text = """<tr class="highlight">
                        <td width="10" class="align-right">%s</td>
                        <td class="align-left">%s<br /><textarea name="%s" rows="3" cols="50"></textarea></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                        <td width="10" class="align-center"></td>
                    </tr>"""
        # get HTML
        page = ''
        i = 0
        for q in questions :
        #    if i%5==0 :
            if i==0 :
                page += header
            i+=1
            if q['type'] =='radio5' :
                page += radio5 % (i, self.htmlEncode(q['text']), q['q_id'], q['q_id'], q['q_id'], q['q_id'], q['q_id'], q['q_id'])
            if q['type'] == 'text' :
                page += text % (i, self.htmlEncode(q['text']), q['q_id'])
            if q['type'] == 'title' :
                page += title % self.htmlEncode(q['text'])
                i=0
        return page
        
    
    security.declareProtected(Permissions.access_contents_information, 'saveAnswers')
    def saveAnswers(self):
        """saves the submitted questionnaire"""
        language = self.getLanguage()
        formData = self.REQUEST.form
        lecture = formData['lecture']
        type = formData['type']
        # do not save answers if evaluation is closed
        lecture = self.getLectureWithID(lecture, type)
        if lecture['open'] == 'N' :
            return '<p class="warning">Diese Evaluation ist geschlossen!<br />&larr; <a href="index" tal:attributes="href here/questions_index/absolute_url">Zurück zum Fragenkatalog</a></p>'
        # do not save answers if not submitted from a student
        if not self.isStudent() :
            return '<p class="warning">Da Sie keine Berechtigung haben, die Evaluation auszufüllen, können Sie die Fragen nur ansehen, aber nicht beantworten!<br />&larr; <a href="javascript:history.back()">Zurück zum Fragenkatalog</a></p>'
        # validate form data
        requiredQuestions = self.getRequiredQuestions(formData['type'],formData['lecture'])
        for id in [x['q_id'] for x in requiredQuestions] :
            if str(id) not in formData.keys() : 
                if language == 'de' :
                    return '<p>Bitte beantworten Sie alle Fragen (Textfragen sind fakultativ).<br />&larr; <a href="javascript:history.back()">Zurück zum Fragenkatalog</a></p>'
                else :
                    return '<p>Please answer all questions (text questions are optional).<br />&larr; <a href="javascript:history.back()">Back</a></p>'
        # all questions answered -> write form data
        for id in formData :
            if (id != 'lecture') and (id != 'type') and formData[id] :
                self._submitAnswer(id,formData['lecture'],formData[id])
        if language == 'de' :
            return '<p>Besten Dank für das Beantworten der Fragen! Bitte gehen Sie zurück auf die <a href="'+self.questions_index.absolute_url()+'">Hauptseite</a> und beantworten Sie die Fragen zu den anderen Vorlesungen.</p>'
        else :
            return '<p>Thank you for answering the questions! Please return to the <a href="'+self.questions_index.absolute_url()+'">main page</a> and continue with the other lectures.</p>'

    security.declarePrivate('_submitAnswer')
    def _submitAnswer(self, q_id, l_id, value):
        """saves the answer to the database"""
        allCourses = self.isStudent()
        if not allCourses :
            return None
        mainCourse = 'MSc' in allCourses and 'MSc' or 'BSc'
        query = """INSERT INTO `%s`.`q_answers` 
                   (q_id, u_id, l_id, value, course) 
                   VALUES 
                   (%d, %s, %s, %s, %s)
                """ % (self.getDatabase(), int(q_id), self._escape(self.getUser()), self._escape(l_id), self._escape(value), self._escape(mainCourse))
        self._queryString(query)
    
    security.declareProtected(Permissions.access_contents_information, 'getLecturesForSemester')
    def getLecturesForSemester(self, semester=None, course=None):
        """returns the lectures for the newest semester (e.g. 2006W) and for the given (student)semester (1, 2, 3, etc.) and course (MSc or BSc)
           if no semester is passed it shows lectures for all (student)semesters
           if no course is passed it shows lectures for all courses"""
        if semester != None :
            semesterWhere = "AND semester=%s" % self._escape(semester)
        else :
            semesterWhere = ''
        if course != None :
            courseWhere = "AND course=%s" % self._escape(course)
        else :
            courseWhere = ""
        query = """SELECT * 
                    FROM `%s`.`lectures` 
                    WHERE semkez=%s 
                      %s 
                      %s
                    ORDER BY name""" % (self.getDatabase(), self._escape(self.getCurrentSemester()[0]), semesterWhere, courseWhere)
        return self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'getLecturesNotForSemester')
    def getLecturesNotForSemester(self, course=None):
        """returns the lectures for the newest semester (e.g. 2006W) not for the semester of the user and for the given course (MSc or BSc)
           if no course is passed it shows lectures for all courses"""
        if course != None :
            where = "AND course=%s" % self._escape(course)
        else :
            where = ""
        query = """SELECT * 
                    FROM `%s`.`lectures` 
                    WHERE semester!=%s 
                      AND semkez=%s 
                      %s
                    ORDER BY name""" % (self.getDatabase(), self._escape(self.getSemesterOfStudent()), self._escape(self.getCurrentSemester()[0]), where)
        return self._queryString(query)
        
    security.declareProtected(Permissions.access_contents_information, 'getAnswersForLecture')
    def getAnswersForLecture(self, lecture):
        """returns the answers for the given lecture and the logged in user"""
        query = "SELECT q_id, value FROM `%s`.`q_answers` WHERE u_id=%s AND l_id=%s" % (self.getDatabase(), self._escape(self.getUser()), self._escape(lecture))
        return self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'getSemesterWithSemkez')
    def getSemesterWithSemkez(self, semkez):
        """returns the status for general questions for semester semkez (used to check if general questions for this semester are open)"""
        query = """SELECT * FROM `%s`.`general_questions` WHERE `semkez`=%s""" % (self.getDatabase(), self._escape(semkez))
        return self._queryString(query)
    
    security.declareProtected(Permissions.access_contents_information, 'getLectureWithID')
    def getLectureWithID(self, id, type=None):
        """returns the lecture with the given id and type (general or lecture)
           if type==general and semester is not yet in the table "general_questions" a corresponding dictionary is built and returned
           if no type is given, it is guessed from the id"""
        if type == None :
            type = 'lecture'
            try :
                id = int(id)
            except :
                if len(id) == 5 :
                    type = 'general' # if it contains characters and is of length 5 it's type==general (e.g. 2006W) (maybe lecture id will also contain characters some day so we check also the length)
        if type == 'general' :
            lecture = self.getSemesterWithSemkez(id)
            if lecture :
                open = lecture[0]['open']
                public = lecture[0]['public']
                date_opened = lecture[0]['date_opened']
                date_closed = lecture[0]['date_closed']
            else :
                open = 'N'
                public = 'N'
                date_opened = None
                date_closed = None
            semlang = self.getSemlangForSemkez(id)
            return {'name':'Allgemeine Fragen', 'nameEN':'General Questions', 'lecturer':'', 'code':'', 'semkez':id, 'semlang':semlang, 'open':open, 'public':public, 'date_opened':date_opened, 'date_closed':date_closed, 'q_set':self.getQuestionSetForSemester(id), 'l_id':id}
        else :
            query = "SELECT * FROM `%s`.`lectures` WHERE l_id=%s" % (self.getDatabase(), self._escape(id))
            return self._queryString(query)[0]

    security.declareProtected(Permissions.access_contents_information, 'getQuestionSetForSemester')
    def getQuestionSetForSemester(self, semkez):
        """returns the question set for general questions for the given semester"""
        query = "SELECT q_set FROM `%s`.`general_questions` WHERE semkez=%s" % (self.getDatabase(), self._escape(semkez))
        result = self._queryString(query)
        if result :
            return result[0]['q_set']
        else :
            return 'general'

    security.declareProtected(Permissions.access_contents_information, 'getQuestions')
    def getQuestions(self, type, lecture):
        """returns the question for the given type and lecture (returns only enabled questions)"""
        if type == 'general' :
            set = self.getQuestionSetForSemester(semkez=lecture)
            query = """SELECT q_types.q_id, q_texts.text, q_types.type, q_types.a_set 
                        FROM `%s`.`q_types` 
                        LEFT JOIN `%s`.`q_texts` USING(q_id) 
                        WHERE q_types.q_set=%s AND q_texts.language=%s AND q_types.enabled=1 
                        ORDER BY sort""" % (self.getDatabase(), self.getDatabase(), self._escape(set), self._escape(self.getLanguage()))
        else :
            query = """SELECT q_types.q_id, q_texts.text, q_types.type, q_types.a_set 
                        FROM `%s`.`lectures` 
                        LEFT JOIN `%s`.`q_types` USING(q_set) 
                        LEFT JOIN `%s`.`q_texts` USING(q_id) 
                        WHERE q_types.q_id IS NOT NULL AND lectures.l_id=%s AND q_texts.language=%s AND q_types.enabled=1 
                        ORDER BY sort""" % (self.getDatabase(), self.getDatabase(), self.getDatabase(), self._escape(lecture), self._escape(self.getLanguage()))
        questions = self._queryString(query)
        if not questions :
            return []
        return questions

    security.declareProtected(Permissions.access_contents_information, 'getRequiredQuestions')
    def getRequiredQuestions(self, type, lecture):
        """returns the required questions for the given lecture and type"""
        questions = self.getQuestions(type,lecture)
        requiredQuestions = []
        for q in questions :
            if q['type'] in ('radio5') :
                requiredQuestions.append(q)
        return requiredQuestions


## evaluation_ methods
    security.declareProtected(Permissions.access_contents_information, 'getAllQuestions')
    def getAllQuestions(self, type, lecture):
        """returns the question for the given type and lecture (returns all questions, even disabled questions)"""
        if type == 'general' :
            set = self.getQuestionSetForSemester(semkez=lecture)
            query = """SELECT q_types.q_id, q_texts.text, q_types.type, q_types.a_set 
                        FROM `%s`.`q_types` 
                        LEFT JOIN `%s`.`q_texts` USING(q_id) 
                        WHERE q_types.q_set=%s AND q_texts.language=%s 
                        ORDER BY sort""" % (self.getDatabase(), self.getDatabase(), self._escape(set), self._escape(self.getLanguage()))
        else :
            query = """SELECT q_types.q_id, q_texts.text, q_types.type, q_types.a_set 
                        FROM `%s`.`lectures` 
                        LEFT JOIN `%s`.`q_types` USING(q_set) 
                        LEFT JOIN `%s`.`q_texts` USING(q_id) 
                        WHERE q_types.q_id IS NOT NULL AND lectures.l_id=%s AND q_texts.language=%s 
                        ORDER BY sort""" % (self.getDatabase(), self.getDatabase(), self.getDatabase(), self._escape(lecture), self._escape(self.getLanguage()))
        questions = self._queryString(query)
        if not questions :
            return []
        return questions

    security.declareProtected(Permissions.access_contents_information, 'getNumberOfAnswers')
    def getNumberOfAnswers(self, type, lecture, course=None):
        """returns the number of answers for the given type, lecture and course
           course is only necessary and used if type==general"""
        # check privileges
        if not self.isPublicEvaluator() :
            return None
        if not self.isEvaluator() and self.getLectureWithID(lecture,type)['public'] != 'Y' :
            return None
        questions = self.getRequiredQuestions(type,lecture)
        if not questions :
            questions = self.getAllQuestions(type,lecture)
        if questions :
            if type == 'general' :
                return len(self.getAnswersForLectureAndQuestion(lecture,questions[0]['q_id'],course))
            else :
                return len(self.getAnswersForLectureAndQuestion(lecture,questions[0]['q_id']))
        else :
            return 0

    security.declareProtected(Permissions.access_contents_information, 'getAnswersForLectureAndQuestion')
    def getAnswersForLectureAndQuestion(self, lecture, question, course=None):
        """returns the answers for the given lecture, question and course
           if course is not given it returns answers for all courses (BSc and/or MSc) (course only necessary for type==general)"""
        # check privileges
        if not self.isPublicEvaluator() :
            return None
        if not self.isEvaluator() and self.getLectureWithID(lecture)['public'] != 'Y' :
            return None
        query = """SELECT `value` 
                   FROM `%s`.`q_answers` 
                   WHERE l_id=%s AND q_id=%d""" % (self.getDatabase(), self._escape(lecture), int(question))
        if course != None :
            query += " AND course=%s" % self._escape(course)
        return self._queryString(query)
        
    security.declareProtected(Permissions.access_contents_information, 'getSummaryForLectureAndQuestion')
    def getSummaryForLectureAndQuestion(self, lecture, question, course=None):
        """returns the summary for the given lecture, question and course
           course is only necessary if type==general"""
        # check privileges
        if not self.isPublicEvaluator() :
            return None
        if not self.isEvaluator() and self.getLectureWithID(lecture)['public'] != 'Y' :
            return None
        query = """SELECT value 
                   FROM `%s`.`q_summaries` 
                   WHERE l_id=%s 
                        AND q_id=%d""" % (self.getDatabase(), self._escape(lecture), int(question))
        if course != None :
            query += ' AND course=%s'% self._escape(course)
        result = self._queryString(query)
        try:
            sum = result[0]['value']
        except Exception, e:
            sum = ''
        return sum

    security.declareProtected(Permissions.access_contents_information, 'getAnswersForSet')
    def getAnswersForSet(self, set):
        """returns the possible answers for the given set (text values for answers 1-5)"""
        query = "SELECT text FROM `%s`.`a_sets` WHERE a_set=%s ORDER BY value" % (self.getDatabase(), self._escape(set))
        return [x['text'] for x in self._queryString(query)]

    security.declareProtected(Permissions.access_contents_information, 'getLectures')
    def getLectures(self, semkez=None, course=None):
        """returns the lectures for the given semester (semkez) and course (MSc or BSc)
           if no semester is given it shows lectures for all semesters
           if no course is given it shows lectures for all courses"""
        args = []
        # check privileges
        if not self.isPublicEvaluator() :
            return None
        if not self.isEvaluator() :
            args.append("public='Y'")
        if semkez :
            args.append('semkez=%s' % self._escape(semkez))
        if course :
            args.append('course=%s' % self._escape(course))
        if args :
            where = 'WHERE %s' % (' AND '.join(args))
        else :
            where = ''
        query = """SELECT * 
                    FROM `%s`.`lectures` 
                    %s
                    ORDER BY semkez, course, name""" % (self.getDatabase(), where)
        return self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'getStudentInfoForLecture')
    def getStudentInfoForLecture(self, lecture, course=None):
        """returns the student info for the given lecture and course (sex, language, department)
           if no course is given it shows student info for all courses"""
        # check privileges
        if not self.isPublicEvaluator() :
            return None
        if not self.isEvaluator() and self.getLectureWithID(lecture)['public'] != 'Y' :
            return None
        if course :
            where = 'AND course=%s' % self._escape(course)
        else :
            where = ''
        query = """SELECT sex, language, department 
                    FROM `%s`.`student_info` 
                    LEFT JOIN `%s`.`q_answers` USING (u_id) 
                    WHERE l_id=%s
                        %s
                    GROUP BY `student_info`.u_id""" % (self.getDatabase(), self.getDatabase(), self._escape(lecture), where)
        return self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'getEvaluationHTML')
    def getEvaluationHTML(self):
        """returns the html for the evaluation"""
        type = self.REQUEST.type
        lecture = self.REQUEST.lecture
        course = self.REQUEST.course
        # check privileges
        if not self.isPublicEvaluator() :
            return '<p class="warning">Sie haben keine Berechtigung, diese Evaluation anzuschauen!</p>'
        if not self.isEvaluator() and self.getLectureWithID(lecture)['public'] != 'Y' :
            return '<p class="warning">Sie haben keine Berechtigung, diese Evaluation anzuschauen!</p>'
        # html templates
        table = """<table>
                       <col width="10" class="align-right" />
                       <col class="align-left" />
                   <tbody>
                       %s
                   </tbody>
                   </table>"""
        title = """<tr class="rowheading">
                        <td width="10" class="align-left" colspan="2">
                           <br /><b class="large">%s</b>
                        </td>
                   </tr>"""
        text = """<tr>
                       <td width="10" class="align-right">%d</td>
                       <td class="align-left">
                           <b>%s</b><br />
                           %s
                           <ul id="txt%s" class="expandable">
                               %s
                           </ul>
                           %s
                           %s
                       </td>
                   </tr>"""
        answersTitle = """<strong>Antworten <a href="javascript:toggleElement(%%s)" class="no-print" title="Antworten zeigen"><img id="img%%s" src="%s/plus_gif" alt="Answers Show/Hide" /></a></strong><br />""" % self.absolute_url()
        noAnswersTitle = """Keine Antworten vorhanden.<br />"""
        summaryTitle = """<strong>Zusammenfassung</strong><br />"""
        answersTemplate = """<li>%s</li>"""
        radio5 = """<tr>
                        <td width="10" class="align-right">%d</td>
                        <td class="align-left">
                            <b>%s</b>
                            <table>
                                %s
                            </table>
                        </td>
                    </tr>"""
        radioTemplate = """<tr class="highlight">
                                <td style="width:200px; text-align:left; vertical-align:middle;">%s&nbsp;&nbsp;</td>
                                <td style="width:30px; text-align:right;">&nbsp;%d</td>
                                <td style="width:50px; text-align:right;">&nbsp;%s</td>
                                <td style="width:10px; text-align:left;">&nbsp;</td>
                                <td style="width:auto; text-align:left;">%s</td>
                           </tr>"""
        barTemplate = """<img src="%s/bar_gif" width="%%d" height="6" alt='' title='' /><img src="%s/bar_g_gif" width="%%d" height="6" alt='' title='' /><img src="%s/bar_t_gif" width="%%d" height="6" alt='' title='' /><img src="%s/bar_b_gif" width="1" height="6" alt='' title='' />""" % ((self.absolute_url(),)*4)
        # get evaluation data
        questions = self.getAllQuestions(type=type,lecture=lecture)
        rows = ''
        i = 0
        for q in questions :
            i += 1
            if type == 'general' :
                answers = self.getAnswersForLectureAndQuestion(lecture,q['q_id'],course)
            else :
                answers = self.getAnswersForLectureAndQuestion(lecture,q['q_id'])
            if q['type'] =='radio5' :
                html = ''
                k = 0
                answerSet = self.getAnswersForSet(q['a_set'])+['keine Antwort']
                answerValues = [x['value'] for x in answers]
                for a in range(1,6)+[0] :
                    num = answerValues.count(str(a))
                    numNoAnswer = answerValues.count('0')
                    numTotal = len(answerValues)
                    if numTotal :
                        percent = '(%d%%)' % (round(float(num)/numTotal*100,0))
                    else :
                        percent = ''
                    if numTotal and a :
                        # bar is 180px long, 1px is for black end line
                        w1 = round(float(num)/numTotal*179,0)
                        w3 = round(float(numNoAnswer)/numTotal*179,0)
                        w2 = 179-w1-w3
                        bar = barTemplate % (w1, w2, w3)
                    else :
                        bar = ''
                    html += radioTemplate % (bar, num, percent, self.htmlEncode(answerSet[k]))
                    k += 1
                rows += radio5 % (i, self.htmlEncode(q['text']), html)
            if q['type'] == 'text' :
                if type == 'general' :
                    sum = self.getSummaryForLectureAndQuestion(lecture,q['q_id'],course)
                else :
                    sum = self.getSummaryForLectureAndQuestion(lecture,q['q_id'])
                if answers :
                    at = answersTitle % (q['q_id'], q['q_id'])
                else :
                    at = noAnswersTitle
                if sum :
                    st = summaryTitle
                else :
                    st = ''
                aa = '\n'.join([answersTemplate % self.htmlEncode(a['value']) for a in answers])
                rows += text % (i, self.htmlEncode(q['text']), at, q['q_id'], aa, st, self.htmlEncode(sum))
            if q['type'] == 'title' :
                rows += title % q['text']
                i = 0
        return table % rows

    security.declareProtected(Permissions.access_contents_information, 'getCompareListHTML')
    def getCompareListHTML(self, type):
        """returns the HTML code for the compare list for the given type (general/lecture)"""
        # check privileges
        if not self.isPublicEvaluator() :
            return None
        subheader = """<p><strong>%s</strong></p>"""
        list = """<input type="checkbox" name="ids:list" id="id_%s" value="%s" /> <label for="id_%s">%s (%d)</label><br />"""
        page = ''
        courses = ['BSc','MSc']
        if type == 'general' :
            # get general questions
            general = {}
            for sem in self.getAllSemesters() :
                if self.isEvaluator() or self.getLectureWithID(sem[0])['public'] == 'Y' :
                    page += subheader % sem[1]
                    for course in courses :
                        id = '%s_%s' % (course, sem[0])
                        text = 'Allgemeine Fragen %s' % course
                        answers = self.getNumberOfAnswers('general',sem[0],course)
                        page += list % (id, id, id, self.htmlEncode(text), answers)
        elif type == 'lecture' :
            # get lectures
            lectures = {'BSc':{},'MSc':{}}
            semlangs = {}
            for course in courses :
                ## group the lectures by semesters
                for lecture in self.getLectures(None,course) :
                    if self.isEvaluator() or lecture['public']=='Y' :
                        if not lectures[course].has_key(lecture['semkez']) :
                            lectures[course][lecture['semkez']] = []
                        text = '%s: %s (%s)' % (lecture['code'], lecture['name'], lecture['lecturer'])
                        lectures[course][lecture['semkez']].append({'l_id':lecture['l_id'], 'text':text, 'answers':self.getNumberOfAnswers('lecture',lecture['l_id'],course)})
                        semlangs[lecture['semkez']] = lecture['semlang']
            ## sort the dict
            semesters = semlangs.keys()
            semesters.sort()
            # semesters.reverse()
            ## output the lectures
            for semester in semesters :
                page += subheader % self.htmlEncode(semlangs[semester])
                for course in courses :
                    if lectures[course].has_key(semester) :
                        page += subheader % self.htmlEncode(course)
                        for lecture in lectures[course][semester] :
                            page += list % (lecture['l_id'], lecture['l_id'], lecture['l_id'], self.htmlEncode(lecture['text']), lecture['answers'])
        # return the page
        return page

    security.declareProtected(Permissions.access_contents_information, 'getLecturesForCompare')
    def getLecturesForCompare(self):
        """returns the selected lectures for the evaluation compare view (taking ids from request)
           checks also if lectures have same questionnaire"""
        # check privileges
        if not self.isPublicEvaluator() :
            return None
        ids = self.REQUEST.ids
        type = self.REQUEST.type
        set = ''
        # get lecture names
        lectures = []
        for id in ids :
            # fix id for type==general (e.g. "BSc_2006W")
            if type == 'general' :
                course = id[:3]
                id = id[4:]
            lecture = self.getLectureWithID(id)
            if type == 'general' :
                lecture['course'] = course
            # check if lecture has different questionnnaire
            if not set :
                set = lecture['q_set']
            if lecture['q_set'] != set :
                diff = 1
            else :
                diff = 0
            # append lecture
            if type == 'lecture' :
                name = '%s: %s (%s) (%s %s) (%s ausgefüllte Fragenkataloge)' % (lecture['code'], lecture['name'], lecture['lecturer'], lecture['semlang'], lecture['course'], self.getNumberOfAnswers(type,id,lecture['course']))
            else :
                name = '%s (%s %s) (%s ausgefüllte Fragenkataloge)' % (lecture['name'], lecture['semlang'], lecture['course'], self.getNumberOfAnswers(type,id,lecture['course']))
            lectures.append((diff,name,type,id,lecture['course']))
        return lectures

    security.declareProtected(Permissions.access_contents_information, 'getCompareHTML')
    def getCompareHTML(self):
        """returns the HTML code for the evaluation compare view"""
        # check privileges
        if not self.isPublicEvaluator() :
            return '<p class="warning">Sie haben keine Berechtigung, diese Evaluation anzuschauen!</p>'
        ids = self.REQUEST.ids
        type = self.REQUEST.type
        # get lectures, question ids (in case they are not using the same question sets) and questions for first lecture (others should be equal)
        lectures = []
        qids = {}
        i = 0
        for id in ids :
            # fix id for type==general (e.g. "BSc_2006W")
            if type == 'general' :
                course = id[:3]
                id = id[4:]
            # get lectures
            lecture = self.getLectureWithID(id)
            if type == 'general' :
                lecture['course'] = course
            lectures.append(lecture)
            # get question ids
            qq = self.getAllQuestions(type,id)
            qids[id] = [x['q_id'] for x in qq]
            # get questions for first lecture
            if not i :
                questions = qq
            i += 1
        # check privileges for each lecture
        if not self.isEvaluator() :
            for lecture in lectures :
                if lecture['public'] != 'Y' :
                    return '<p class="warning">Sie haben keine Berechtigung, diese Evaluation anzuschauen!</p>'
        # templates
        title = """<tr class="rowheading">
                        <td width="10" class="align-left" colspan="2">
                            <br /><b class="large">%s</b>
                        </td>
                   </tr>"""
        text = """<tr>
                       <td width="10" class="align-right">%d</td>
                       <td class="align-left">
                           <b>%s</b>
                              <table>
                                   %s
                              </table>
                       </td>
                   </tr>"""
        textTemplate = """<tr class="highlight">
                              <td width="311" class="align-left" valign="middle">%s</td>
                              <td width="25" class="align-right">%d</td>
                              <td width="5" class="align-left"></td>
                              <td class="align-left">%s%s</td>
                            </tr>"""
        radio5 = """<tr>
                          <td width="10" class="align-right">%d</td>
                          <td class="align-left">
                              <b>%s</b>
                                <table>
                                  %s
                                </table>
                          </td>
                      </tr>"""
        radio5Template = """<tr class="highlight">
                                <td width="200" class="align-left" valign="middle">
                                    %s
                                </td>
                                <td width="105" class="align-left">%s</td>
                                <td width="25" class="align-right">%d</td>
                                <td width="5" class="align-left"></td>
                                <td class="align-left">%s%s</td>
                            </tr>"""
        linkTemplate = """ <a href="%s?type=%s&lecture=%s&course=%s" title="Detailansicht" target="_blank" class="no-print">&raquo;&raquo;</a>"""
        barTemplate = """<img src="%s/bar_gif" width="%%d" height="6" alt='' title='' /><img src="%s/bar_b_gif" width="1" height="6" alt='' title='' /><img src="%s/bar_gif" width="%%d" height="6" alt='' title='' /><img src="%s/bar_g_gif" width="%%d" height="6" alt='' title='' /><img src="%s/bar_b_gif" width="1" height="6" alt='' title='' /><img src="%s/bar_g_gif" width="%%d" height="6" alt='' title='' />""" % ((self.absolute_url(),)*6)
        averageBarTemplate = """<img src="%s/bar_a_gif" width="%%d" height="6" alt='' title='' /><img src="%s/bar_g_gif" width="%%d" height="6" alt='' title='' />""" % ((self.absolute_url(),)*2)
        # generate page
        page = ''
        i = 0
        index = 0
        for q in questions :
            i+=1
            if q['type'] =='radio5' :
            # radio5 evaluation form
                values = []
                rows = ''
                for lecture in lectures :
                    # try getting answers, failure when no corresponding id in qids (actually in this case the question set is not corresponding)
                    try :
                        if type == 'general' :
                            answers = self.getAnswersForLectureAndQuestion(lecture['l_id'],qids[str(lecture['l_id'])][index],lecture['course'])
                        else :
                            answers = self.getAnswersForLectureAndQuestion(lecture['l_id'],qids[str(lecture['l_id'])][index])
                    except :
                        answers = []
                    # try filtering answers, failure when it is a text answer
                    try :
                        answers = [float(x['value']) for x in answers if int(x['value']) != 0]
                    except :
                        answers = []
                    num = len(answers)
                    value = self.getMedian(answers)
                    if num :
                        values.append((value[0],num))
                    # generate HTML
                    if num :
                        w1 = int((value[1]-1)/4*178)
                        w2 = int((value[0]-1)/4*178)-w1
                        w3 = int((value[2]-1)/4*178)-w1-w2
                        w4 = 178-w1-w2-w3
                        bar = barTemplate % (w1, w2, w3, w4)
                        val = '%.1f(-%.1f/+%.1f)' % (round(value[0],1), round(value[0]-value[1],1), round(value[2]-value[0],1))
                    else :
                        bar = ''
                        val = ''
                    if type == 'lecture' :
                        name = '%s: %s (%s %s)' % (lecture['code'], lecture['name'], lecture['semlang'], lecture['course'])
                    else :
                        name = '%s (%s %s)' % (lecture['name'], lecture['semlang'], lecture['course'])
                    link = linkTemplate % (self.evaluation_view.absolute_url(), type, lecture['l_id'], lecture['course'])
                    rows += radio5Template % (bar, val, num, self.htmlEncode(name), link)
                # get average for all lectures
                averageValues = [x[0] for x in values]
                num = sum([x[1] for x in values])
                value = self.getAverage(averageValues)
                if num :
                    w1 = int((value-1)/4*180)
                    w2 = 180-w1
                    bar = averageBarTemplate % (w1, w2)
                    val = '%.1f' % (round(value,1))
                else :
                    bar = ''
                    val = ''
                name = 'Arithmetisches Mittel der Vorlesungen'
                link = ''
                rows += radio5Template % (bar, val, num, self.htmlEncode(name), link)
                page += radio5 % (i, self.htmlEncode(q['text']), rows)
            if q['type'] == 'text' :
            # text evaluation form
                answers = []
                rows = ''
                for lecture in lectures :
                    # try getting summary, failure when no corresponding id in qids (actually in this case the question set is not corresponding)
                    try :
                        if type == 'general' :
                            summary = self.getSummaryForLectureAndQuestion(lecture['l_id'],qids[str(lecture['l_id'])][index],lecture['course'])
                        else :
                            summary = self.getSummaryForLectureAndQuestion(lecture['l_id'],qids[str(lecture['l_id'])][index])
                        if not summary :
                            summary = 'Keine Zusammenfassung verfügbar.'
                    except :
                        summary = 'Keine Zusammenfassung verfügbar.'
                    # try getting number of answers, failure when no corresponding id in qids (actually in this case the question set is not corresponding)
                    try :
                        if type == 'general' :
                            num = len(self.getAnswersForLectureAndQuestion(lecture['l_id'],qids[str(lecture['l_id'])][index],lecture['course']))
                        else :
                            num = len(self.getAnswersForLectureAndQuestion(lecture['l_id'],qids[str(lecture['l_id'])][index]))
                    except :
                        num = 0
                    # generate HTML
                    if type == 'lecture' :
                        name = '%s: %s (%s %s)' % (lecture['code'], lecture['name'], lecture['semlang'], lecture['course'])
                    else :
                        name = '%s (%s %s)' % (lecture['name'], lecture['semlang'], lecture['course'])
                    link = linkTemplate % (self.evaluation_view.absolute_url(), type, lecture['l_id'], lecture['course'])
                    rows += textTemplate % (self.htmlEncode(summary), num, self.htmlEncode(name), link)
                page += text % (i, q['text'], rows)
            if q['type'] == 'title' :
            # title
                page += title % q['text']
                i = 0
            index += 1
        return page

    security.declareProtected(Permissions.access_contents_information, 'getMedian')
    def getMedian(self, values):
        """returns median and quantils for the passed values as tuple (median,firstQuantil,fourthQuantil)"""
        if not values :
            return (0,0,0)
        # median method
        def median(values) :
        # calculate median
            values.sort()
            num = len(values)
            if num%2 :
            # odd number of values
                mid = (num-1)/2
                med = values[num/2]
            else :
            # even number of values
                mid = num/2
                med = (values[mid-1]+values[mid])/2
            return med
        # quantil method
        def quantil(values,quantil) :
        # calculate quantil, e.g. quantil=0.25 -> quartil
            values.sort()
            num = len(values)
            i = num*quantil
            if float(int(i))==i :
            # i is int
                i = int(i)
                qu = (values[i-1]+values[i])/2
            else :
            # i is float
                i = int(i)
                qu = values[i]
            return qu
        # interpolated quantil method
        def interpolatedQuantil(values,quantil) :
        # calculate interpolated median
            num = len(values)
            numLess = len([x for x in values if x<quantil])
            numEqual = len([x for x in values if x==quantil])
            if numEqual :
                ipQuantil = quantil - 0.5 + (0.5*num-numLess)/numEqual
            else :
                ipQuantil = quantil
            return ipQuantil
        # main method
        med = median(values)
        med = interpolatedQuantil(values,med)
        qu = quantil(values,1.0/6)
        qu2 = quantil(values,1.0/6*5)
        return (med,qu,qu2)

    security.declareProtected(Permissions.access_contents_information, 'getAverage')
    def getAverage(self, values):
        """returns the average for the passed values"""
        if not values :
            return 0
        return float(sum(values))/len(values)

    security.declareProtected(Permissions.access_contents_information, 'exportAsExcel')
    def exportAsExcel(self):
        """exports the evaluation compare view as excel CSV"""
        # check privileges
        if not self.isPublicEvaluator() :
            return None
        type = self.REQUEST.type
        ids = self.REQUEST.ids
        os = self.REQUEST.os
        if os == 'mac' :
            encoding = 'mac-roman'
            lineEnd = '\r'
        elif os == 'win' :
            encoding = 'latin-1'
            lineEnd = '\r\n'
        # get lectures, question ids (in case they are not using the same question sets) and questions for first lecture (others should be equal)
        lectures = []
        qids = {}
        i = 0
        for id in ids :
            # fix id for type==general (e.g. "BSc_2006W")
            if type == 'general' :
                course = id[:3]
                id = id[4:]
            # get lectures
            lecture = self.getLectureWithID(id)
            if type == 'general' :
                lecture['course'] = course
            lectures.append(lecture)
            # get question ids
            qq = self.getAllQuestions(type,id)
            qq = [q for q in qq if q['type']=='radio5']
            qids[id] = [x['q_id'] for x in qq]
            # get questions for first lecture
            if not i :
                questions = qq
            i += 1
        # check privileges for each lecture
        if not self.isEvaluator() :
            for lecture in lectures :
                if lecture['public'] != 'Y' :
                    return None
        # generate csv body
        csvBody = ''
        values = {}
        numAll = []
        for lecture in lectures :
            if type == 'general' :
                num = self.getNumberOfAnswers(type,lecture['l_id'],lecture['course'])
            else :
                num = self.getNumberOfAnswers(type,lecture['l_id'])
            numAll.append(int(num))
            # fix wrong lecturer format (line breaks)
            lecturer = lecture['lecturer'].replace('\r',' / ')
            lecturer = lecturer.replace('\n','')
            csvBody += '"%s",%s,%s,"%s","%s",%s,Mittelwert' % (lecture['semlang'], lecture['course'], lecture['code'], lecture['name'], lecturer, num)
            index = 0
            for q in questions :
                # try getting answers, failure when no corresponding id in qids (actually in this case the question set is not corresponding)
                try :
                    if type == 'general' :
                        answers = self.getAnswersForLectureAndQuestion(lecture['l_id'],qids[str(lecture['l_id'])][index],lecture['course'])
                    else :
                        answers = self.getAnswersForLectureAndQuestion(lecture['l_id'],qids[str(lecture['l_id'])][index])
                except :
                    answers = []
                try :
                # failure when it is a text answer
                    answers = [float(x['value']) for x in answers if int(x['value']) != 0]
                except :
                    answers = []
                value = self.getMedian(answers)[0]
                if not values.has_key(q['q_id']) :
                    values[q['q_id']] = []
                if len(answers) :
                    values[q['q_id']].append(value)
                    value = round(value,1)
                else :
                    value = '-'
                csvBody += ',%s' % value
                index += 1
            csvBody += lineEnd
        # generate csv header
        csvHeader = 'Semester,Bachelor/Master,LE-Code,Vorlesung,Dozent(en),Anzahl Formulare,'
        for q in questions :
            csvHeader += ',"%s"' % q['text']
        csvHeader += lineEnd
        # generate csv averages
        csvAverages = ',,,Werte des Studiengangs,,%s,' % sum(numAll)
        for q in questions :
            if len(values[q['q_id']]) :
                average = self.getAverage(values[q['q_id']])
                average = round(average,1)
            else :
                average = '-'
            csvAverages += ',%s' % average
        csvAverages += lineEnd
        # generat and csv
        csv = csvHeader + csvAverages + csvBody
        now = localtime()
        date = '%d-%02d-%02d' % (now[0], now[1], now[2])
        # send file
        self.REQUEST.RESPONSE.setHeader('Content-type', 'octet-stream/octet-stream')
        self.REQUEST.RESPONSE.setHeader('Content-disposition', 'inline; filename="evaluation_%s.csv"' % date)
        return csv.encode(encoding, 'replace')


## abstracts_ methods
    security.declareProtected(Permissions.access_contents_information, 'getSummaryStatus')
    def getSummaryStatus(self, type, lecture, course=None):
        """returns the status for the summary of the given type, lecture and course (course is only necessary if type==general)
           (True if all summaries are written, False if one or more summaries are not yet written)"""
        # check privileges
        if not self.isTextEvaluator() :
            return None
        answers = self.getSummariesForLecture(lecture,course)
        # questions = self.getTextQuestions(type,lecture)
        # questions = [x for x in questions if x['type']=='text']
        questions = [q for q in self.getAllQuestions(type,lecture) if q['type']=='text']
        return len(answers)==len(questions)
    
    security.declareProtected(Permissions.access_contents_information, 'getSummariesForLecture')
    def getSummariesForLecture(self, lecture, course=None):
        """returns all summaries for the given lecture and course (course is only necessary if type==general)"""
        # check privileges
        if not self.isTextEvaluator() :
            return None
        query = """SELECT `q_id`, `value` 
                   FROM `%s`.`q_summaries` 
                   WHERE l_id=%s""" % (self.getDatabase(), self._escape(lecture))
        if course :
            query += " AND course=%s" % self._escape(course)
        return self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'getTextEvaluationHTML')
    def getTextEvaluationHTML(self):
        """returns the HTML code for the text summaries"""
        # check privileges
        if not self.isTextEvaluator() :
            return None
        type = self.REQUEST.type
        lecture = self.REQUEST.lecture
        course = self.REQUEST.course
        # html templates
        table = """<table>
                       <col width="10" class="align-right" />
                       <col class="align-left" />
                   <tbody>
                       %s
                   </tbody>
                   </table>"""
        text = """<tr>
                      <td width="10" class="align-right">%d</td>
                      <td class="align-left">
                        <b>%s</b>
                        <ul>
                            %s
                        </ul>
                        Zusammenfassung<br />
                        <textarea name="%s" rows="6" cols="60">%s</textarea>
                      </td>
                  </tr>"""
        answersTemplate = """<li>%s</li>"""
        title = """<tr class="rowheading">
                        <td width="10" class="align-left" colspan="2">
                           <br /><b class="large">%s</b>
                        </td>
                   </tr>"""
        # generate HTML
        questions = [q for q in self.getAllQuestions(type,lecture) if q['type'] in ('text','title')]
        page = ''
        i = 0
        for q in questions :
            i+=1
            if type == 'general' :
                answers = self.getAnswersForLectureAndQuestion(lecture,q['q_id'],course)
            else :
                answers = self.getAnswersForLectureAndQuestion(lecture,q['q_id'])
            if q['type'] == 'text' :
                if type == 'general' :
                    sum = self.getSummaryForLectureAndQuestion(lecture,q['q_id'],course)
                else :
                    sum = self.getSummaryForLectureAndQuestion(lecture,q['q_id'])
                aa = '\n'.join([answersTemplate % self.htmlEncode(a['value']) for a in answers])
                page += text % (i, self.htmlEncode(q['text']), aa, 'summaries_%d' % q['q_id'], sum)
            if q['type'] == 'title' :
                page += title % self.htmlEncode(q['text'])
                i = 0
        return table % page

    security.declareProtected(Permissions.access_contents_information, 'saveSummaries')
    def saveSummaries(self):
        """saves the submitted summaries"""
        # check privileges
        if not self.isTextEvaluator() :
            return None
        lecture = self.REQUEST.lecture
        course = self.REQUEST.course
        formData = self.REQUEST.form
        # save summaries
        ids = [(int(x[10:]),x) for x in formData if x[:10] == 'summaries_']
        for id in ids :
            value = formData[id[1]]
            if value :
                query = """REPLACE INTO `%s`.`q_summaries` 
                           (q_id, u_id, l_id, course, value) 
                           VALUES 
                           (%d, %s, %s, %s, %s) """ % (self.getDatabase(), 
                                                       id[0], 
                                                       self._escape(self.getUser()), 
                                                       self._escape(lecture), 
                                                       self._escape(course), 
                                                       self._escape(value))
            else :
                query = """DELETE FROM `%s`.`q_summaries`  
                           WHERE q_id=%d AND l_id=%s AND course=%s""" % (self.getDatabase(), id[0], self._escape(lecture), self._escape(course))
            self._queryString(query)
        return '<p>Zusammenfassungen gespeichert! Bitte gehen Sie zurück auf die <a href="%s">Hauptseite</a>.</p>' % self.abstracts_index.absolute_url()


## admin_ methods
    security.declareProtected(Permissions.access_contents_information, 'getAvailableVVZSemesters')
    def getAvailableVVZSemesters(self):
        """returns all semesters available from VVZ as list of tuples (semkez, semlang)"""
        # check privileges
        if not self.isAdmin() :
            return None
        cache = get_vvz_cache(self)
        if cache :
            semesters = cache.get_semesters()
            semesters.sort()
            return [(sem, cache.getSemesterLong(sem)) for sem in semesters]
        else :
            return []

    security.declareProtected(Permissions.access_contents_information, 'editLectures')
    def editLectures(self):
        """imports/deletes lectures or imports/deletes lecture inscriptions (according to request)
           returns imported lectures/inscriptions if there have been imported any, otherwise None"""
        # check privileges
        if not self.isAdmin() :
            return None
        semesterImport = getattr(self.REQUEST,'semesterImport',None)
        semesterDelete = getattr(self.REQUEST,'semesterDelete',None)
        inscriptionsFile = getattr(self.REQUEST,'inscriptionsFile',None)
        inscriptionsDelete = getattr(self.REQUEST,'inscriptionsDelete',None)
        if semesterImport :
            # import semester
            return {'lectures':self.importLectures(semesterImport)}
        elif semesterDelete :
            # delete semester
            query = """DELETE FROM `%s`.`lectures` WHERE semkez=%s""" % (self.getDatabase(), self._escape(semesterDelete))
            self._queryString(query)
        elif inscriptionsFile:
            # import inscriptions
            return {'inscriptions':self.importInscriptions(inscriptionsFile)}
        elif inscriptionsDelete:
            # delete inscriptions for the given semester
            query = """DELETE FROM `%s`.`inscriptions` WHERE semkez=%s""" % (self.getDatabase(), self._escape(inscriptionsDelete))
            self._queryString(query)
        return None
    
    def importLectures(self, semkez):
        """imports lectures for the given semester from the VVZ and returns the imported lectures"""
        # check privileges
        if not self.isAdmin() :
            return None
        # getLecturers method (this function would be in VVZCache but it's privat)
        def getLecturers(learningUnit):
            lecturers = {}
            main_lecturers = {}
            for event in learningUnit.objectValues((CacheEvent.meta_type, )):
                lecturers.update(event.getLecturers())
                if event.main_lecturer:
                    main_lecturers[event.main_lecturer] = event.main_lecturer
            #second round to place main lecturers' ids in front
            main_ids = main_lecturers.keys()
            adds = [id for id in lecturers.keys() if not id in main_ids]
            #now we get the lecturer objects from the lecturers container
            lecturers = learningUnit.getOwningSemester().getLecturers()
            return [getattr(lecturers, id).title for id in main_ids + adds]
        # getDictForLearningUnit method
        def getDictForLearningUnit(learningUnit):
            # get course code
            if course == self.getBscCourse() :
                courseCode = 'BSc'
            elif course == self.getMscCourse() :
                courseCode = 'MSc'
            # get semester number
            stop = section.title.find('. Semester')
            if stop == -1 :
                semesterNumber = 0
            else :
                semesterNumber = section.title[:stop]
            # get l_id
            semCode = semester.semkez
            semCode = semCode.replace('F','1')
            semCode = semCode.replace('S','1')
            semCode = semCode.replace('H','2')
            semCode = semCode.replace('W','2')
            l_id = int(semCode + learningUnit.getPath().replace('/',''))    # in case that semkez has char other than H/W/F/S this will break, also if path has non int value
            # get lecturers
            lecturers = ' / '.join(getLecturers(learningUnit))
            return {'l_id':l_id, 
                    'code':learningUnit.code, 
                    'semkez':semester.semkez, 
                    'semlang':semester.semKurz, 
                    'semester':semesterNumber, 
                    'course':courseCode, 
                    'name':learningUnit.unit_title['DE'], 
                    'nameEN':learningUnit.unit_title['EN'], 
                    'lecturer':lecturers}
        # main method
        cache = get_vvz_cache(self)
        if not cache :
            return []
        lectures = []
        semester = cache.get_semester(semkez)
        if not semester:
            return []
        for course in (self.getBscCourse(), self.getMscCourse()):
            for section in semester.getCourseByTitle(course).getSections():
                for learningUnit in section.getUnits():
                    # bachelor/master thesis are directly in section (without subsection)
                    lectures.append(getDictForLearningUnit(learningUnit))
                for subsection in section.getSubSections():
                    for learningUnit in subsection.getUnits():
                        lectures.append(getDictForLearningUnit(learningUnit))
        lecturesAdded = []
        for lecture in lectures:
            if lecture['code'] not in [x['code'] for x in lecturesAdded]:
                # import lecture
                query = """REPLACE INTO `%s`.`lectures` 
                              (`l_id`, `code`, `semkez`, `semlang`, `semester`, `course`, `name`, `nameEN`, `lecturer`) 
                           VALUES 
                              (%s, %s, %s, %s, %s, %s, %s, %s, %s)""" % (self.getDatabase(),
                                                                         self._escape(str(lecture['l_id'])),
                                                                         self._escape(str(lecture['code'])),
                                                                         self._escape(str(lecture['semkez'])),
                                                                         self._escape(str(lecture['semlang'])),
                                                                         self._escape(str(lecture['semester'])),
                                                                         self._escape(str(lecture['course'])),
                                                                         self._escape(str(lecture['name'])),
                                                                         self._escape(str(lecture['nameEN'])),
                                                                         self._escape(str(lecture['lecturer']))) # some objects do not seem to be strings, therefore str()
                self._queryString(query)
                lecturesAdded.append(lecture)
        return lecturesAdded

    def importInscriptions(self, inscriptionsFile):
        """imports lecture inscriptions for the current semester and returns the imported inscriptions"""
        # check privileges
        if not self.isAdmin() :
            return None
        data = inscriptionsFile.read().decode('latin1').encode('utf-8')
        inscriptions = []
        i = 0
        for line in data.split('\r\n'):
            if not line:
                continue
            fields = line.split('\t')
            if i==0:
                # first line -> field names
                fieldNames = fields
            else:
                inscription = {}
                inscription['u_id'] = fields[fieldNames.index('LOGINNAME')]
                inscription['code'] = fields[fieldNames.index('VERANSTALTUNGSNUMMER')]
                inscription['name'] = fields[fieldNames.index('VERANSTALTUNGSTITEL')]
                inscription['first_name'] = fields[fieldNames.index('VORNAME')]
                inscription['last_name'] = fields[fieldNames.index('FAMNAME')]
                inscription['email'] = fields[fieldNames.index('EMAILADRESSE')]
                inscriptions.append(inscription)
            i += 1
        query = """DELETE FROM `%s`.`inscriptions` WHERE `semkez`=%s""" % (self.getDatabase(), self._escape(self.getCurrentSemester()[0]))
        self._queryString(query)
        for inscription in inscriptions:
            query = """REPLACE INTO `%s`.`inscriptions` 
                       (`semkez`, `u_id`, `code`, `first_name`, `last_name`, `email`) 
                       VALUES (%s, %s, %s, %s, %s, %s)""" % (self.getDatabase(),
                                                             self._escape(self.getCurrentSemester()[0]),
                                                             self._escape(inscription['u_id']),
                                                             self._escape(inscription['code']),
                                                             self._escape(inscription['first_name']),
                                                             self._escape(inscription['last_name']),
                                                             self._escape(inscription['email']))
            self._queryString(query)
        return inscriptions

    security.declareProtected(Permissions.access_contents_information, 'getLecturesEditor')
    def getLecturesEditor(self):
        """returns a link to the lectures editor (DBEditor) if specified in the management view, otherwise returns None"""
        # check privileges
        if not self.isAdmin() :
            return None
        if self._lecturesEditor:
            if self._lecturesEditor.count('/'):
                folder = self._lecturesEditor.split('/')[0]
                editor = self._lecturesEditor.split('/')[1]
                return getattr(getattr(self,folder,None),editor,None)
            else:
                return getattr(self,self._lecturesEditor,None)
        else :
            return None

    security.declareProtected(Permissions.access_contents_information, 'getLecturesEditorLink')
    def getLecturesEditorLink(self):
        """returns the absolute link to the lectures editor"""
        # check privileges
        if not self.isAdmin() :
            return None
        editor = self.getLecturesEditor()
        if editor :
            return '%s/%s' % (editor.get_container().absolute_url(), editor.id)
        else :
            return ''

    security.declareProtected(Permissions.access_contents_information, 'getStudents')
    def getStudents(self):
        """returns a list of all students as dictionary {'BSc':semesters,'MSc':semesters}
           semesters is a dictionary {'1':students,'2':students,...}
           students is an array of dictionaries {'nethz':,'name':,'status':} (if lectureInscriptionAvailable() is True it has also a key 'lectures' containing the number of lectures the student is inscripted to (including general questions))
           status is the number of answered questionnaires"""
        # check privileges
        if not self.isAdmin() :
            return None
        semkez =  self.getSelectedSemester()
        if self.lectureInscriptionAvailable(semkez):
            students = []
            query = """SELECT `u_id`, `first_name`, `last_name` FROM `%s`.`inscriptions` WHERE `semkez`=%s GROUP BY `u_id`""" % (self.getDatabase(), self._escape(semkez))
            ss = [(x['u_id'],x['first_name'],x['last_name']) for x in self._queryString(query)]
            for (nethz, firstName, lastName) in ss:
                name = '%s %s' % (lastName, firstName)
                # get status
                query = """SELECT COUNT(DISTINCT `q_answers`.`l_id`) AS `count` 
                           FROM `%s`.`q_answers` 
                           LEFT JOIN `%s`.`lectures` USING(`l_id`) 
                           WHERE `u_id`=%s AND (`q_answers`.`l_id`=%s OR `lectures`.`semkez`=%s)""" % (self.getDatabase(), self.getDatabase(), self._escape(nethz), self._escape(semkez), self._escape(semkez))
                status = self._queryString(query)[0][0]
                lectures = len(self.getInscriptionsForStudent(nethz,semkez))+1  # count also general queries
                # append student
                students.append({'nethz':nethz,'name':name,'status':status,'lectures':lectures})
            students.sort(lambda x,y: cmp(x['name'],y['name']))
        else:
            students = {}
            for course in ['BSc','MSc'] :
                ss = getMembersOfGroup(self.getEvaluationGroups()[course])
                for s in ss :
                    student = getLDAP_authEntry(s['PERSID'])[0]
                    # get name and nethz
                    nethz = student['uid'][0]
                    name = student['gecos'][0]
                    # get semester
                    #groups = student['ou']
                    groups = student.get('ou',[])
                    semester = self.getSemesterOfStudent(groups,course=course)
                    # get status
                    query = """SELECT COUNT(DISTINCT `q_answers`.`l_id`) AS `count` 
                               FROM `%s`.`q_answers` 
                               LEFT JOIN `%s`.`lectures` USING(`l_id`) 
                               WHERE `u_id`=%s AND (`q_answers`.`l_id`=%s OR `lectures`.`semkez`=%s)""" % (self.getDatabase(), self.getDatabase(), self._escape(nethz), self._escape(semkez), self._escape(semkez))
                    status = self._queryString(query)[0][0]
                    # append student
                    if not students.has_key(course) :
                        students[course] = {}
                    if not students[course].has_key(semester) :
                        students[course][semester] = []
                    students[course][semester].append({'nethz':nethz,'name':name,'status':status})
        return students

    security.declareProtected(Permissions.access_contents_information, 'saveStatus')
    def saveStatus(self):
        """saves the status for the passed lectures (open/public) and returns success message"""
        # check privileges
        if not self.isAdmin() :
            return None
        # save form
        req = self.REQUEST
        if getattr(req,'saveNow',None) :
            semkez = req.semkez
            open = getattr(req,'open',[])
            public = getattr(req,'public',[])
            # create semester if not exists
            semester = self.getSemesterWithSemkez(semkez)
            if semester :
                semester = semester[0]
            else :
                query = """INSERT INTO `%s`.`general_questions` (`semkez`) VALUES (%s)""" % (self.getDatabase(), self._escape(semkez))
                self._queryString(query)
                semester = self.getSemesterWithSemkez(semkez)[0]
            # save semester
            o = 'general' in open and 'Y' or 'N'
            p = 'general' in public and 'Y' or 'N'
            date = ''
            if o == 'Y' and semester['open'] == 'N':
                date += ', `date_opened`=CURRENT_DATE()'
            if o == 'N' and semester['open'] == 'Y':
                date += ', `date_closed`=CURRENT_DATE()'
            query = """UPDATE `%s`.`general_questions` 
                       SET `open`=%s, `public`=%s
                       %s
                       WHERE `semkez`=%s""" % (self.getDatabase(), self._escape(o), self._escape(p), date, self._escape(semkez))
            self._queryString(query)
            # save lectures
            lectures = self.getLectures(self.getSelectedSemester())
            for lecture in lectures :
                o = str(lecture['l_id']) in open and 'Y' or 'N'
                p = str(lecture['l_id']) in public and 'Y' or 'N'
                date = ''
                if o == 'Y' and lecture['open'] == 'N':
                    date += ', `date_opened`=CURRENT_DATE()'
                if o == 'N' and lecture['open'] == 'Y':
                    date += ', `date_closed`=CURRENT_DATE()'
                query = """UPDATE `%s`.`lectures` 
                           SET `open`=%s, `public`=%s
                           %s
                           WHERE `l_id`=%s""" % (self.getDatabase(), self._escape(o), self._escape(p), date, self._escape(lecture['l_id']))
                self._queryString(query)
            return 'Änderungen gespeichert!'
        return ''
        
    security.declareProtected(Permissions.access_contents_information, 'getAvailableQuestionSets')
    def getAvailableQuestionSets(self):
        """returns the available question sets"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """SELECT DISTINCT q_set
                   FROM `%s`.`q_types`
                   WHERE q_set!="general" AND q_set!="default"
                   ORDER BY q_set""" % self.getDatabase()
        return [x['q_set'] for x in self._queryString(query)]

    security.declareProtected(Permissions.access_contents_information, 'getAvailableAnswerSets')
    def getAvailableAnswerSets(self):
        """returns the available question sets"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """SELECT DISTINCT a_set
                   FROM `%s`.`a_sets`
                   WHERE a_set!="default"
                   ORDER BY a_set""" % self.getDatabase()
        return [x['a_set'] for x in self._queryString(query)]

    security.declareProtected(Permissions.access_contents_information, 'getQuestionsWithAnswerSet')
    def getQuestionsWithAnswerSet(self, set):
        """returns the questions using the given answer set (used to check if an answer set is used)"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """SELECT q_id
                   FROM `%s`.`q_types` 
                   WHERE a_set=%s""" % (self.getDatabase(), self._escape(set))
        return self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'saveAnswerSet')
    def saveAnswerSet(self):
        """saves the passed answer set and returns success message"""
        # check privileges
        if not self.isAdmin() :
            return None
        req = self.REQUEST
        values = {}
        values[1] = getattr(req,'value1',None)
        values[2] = getattr(req,'value2',None)
        values[3] = getattr(req,'value3',None)
        values[4] = getattr(req,'value4',None)
        values[5] = getattr(req,'value5',None)
        aset = req.aset
        if values[1] != None :
            for value in range(1,6) :
                query = """REPLACE INTO `%s`.`a_sets` (a_set, value, text) 
                           VALUES (%s, %d, %s)""" % (self.getDatabase(), self._escape(aset), value, self._escape(values[value]))
                self._queryString(query)
            return 'Antwortkatalog gespeichert!'
        return ''

    security.declareProtected(Permissions.access_contents_information, 'deleteAnswerSet')
    def deleteAnswerSet(self, set):
        """deletes the given answer set and returns success message"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """DELETE FROM `%s`.`a_sets` WHERE a_set=%s""" % (self.getDatabase(), self._escape(set))
        self._queryString(query)
        return 'Antwortkatalog gelöscht!'

    security.declareProtected(Permissions.access_contents_information, 'setLectureSet')
    def setLectureSet(self):
        """sets question set for lecture (passed by request) and returns success message"""
        # check privileges
        if not self.isAdmin() :
            return None
        req = self.REQUEST
        lecture = getattr(req,'lecture')
        type = getattr(req,'type')
        set = getattr(req,'newSet',None)
        if set != None :
            if type == 'general' :
                # create semester if not exists
                if not self.getSemesterWithSemkez(lecture) :
                    query = """INSERT INTO `%s`.`general_questions` (`semkez`) VALUES (%s)""" % (self.getDatabase(), self._escape(lecture))
                    self._queryString(query)
                query = """UPDATE `%s`.`general_questions` SET q_set=%s WHERE semkez=%s""" % (self.getDatabase(), self._escape(set), self._escape(lecture))
            else :
                query = """UPDATE `%s`.`lectures` SET q_set=%s WHERE l_id=%s""" % (self.getDatabase(), self._escape(set), self._escape(lecture))
            self._queryString(query)
            return 'Änderungen gespeichert!'
        return ''

    security.declareProtected(Permissions.access_contents_information, 'getLecturesForEdit')
    def getLecturesForEdit(self):
        """handles questions edit (add/edit/delete/move) and returns dictionary {questions:,message:,editQuestion:}
           questions is a list of dictionaries {id:,type:,de:,en:,a_set:}
           editQuestion is a dictionary {de:,en:,type:,sort:,a_set:}"""
        # check privileges
        if not self.isAdmin() :
            return None
        req = self.REQUEST
        qset = req.qset
        enable = int(getattr(req,'enable',0))
        disable = int(getattr(req,'disable',0))
        qtype = getattr(req,'qtype',None)
        edit = int(getattr(req,'edit',0))
        eqid = int(getattr(req,'eqid',0))
        aset = getattr(req,'aset',None)
        sort = int(getattr(req,'sort',0))
        text = {'de':getattr(req,'de',''),'en':getattr(req,'en','')}
        up = int(getattr(req,'up',0))
        down = int(getattr(req,'down',0))
        message = ''
        editQuestion = {}
        # # delete question
        # if delete :
        #     self.deleteQuestion(delete)
        if enable:
            # enable question
            self.enableQuestion(enable)
        if disable:
            # disable question
            self.disableQuestion(disable)
        # add/modify question
        if qtype :
            if eqid :
                # delete old question
                self.deleteQuestion(eqid)
            # get question sort
            qids = self.getQuestionIds(qset)
            # add question type
            query = """INSERT INTO `%s`.`q_types` (q_id, type, a_set, q_set, sort) 
                       VALUES (%d, %s, %s, %s, %d)""" % (self.getDatabase(), eqid, self._escape(qtype), self._escape(aset), self._escape(qset), sort)
            self._queryString(query)
            if not eqid :
                query = """SELECT LAST_INSERT_ID() AS id"""
                eqid = self._queryString(query)[0]['id']
            # resort
            qids.insert(sort-1,eqid)
            i = 1
            for qid in qids :
                self.moveQuestion(qid,i)
                i+=1
            # add question text
            for language in ['de','en'] :
                if text[language] :
                    query = """INSERT INTO `%s`.`q_texts` 
                               (q_id, language, text) 
                               VALUES 
                               (%d, %s, %s)""" % (self.getDatabase(), eqid, self._escape(language), self._escape(text[language]))
                    self._queryString(query)
        # edit question
        if edit :
            ## get question
            query = """SELECT `q_types`.q_id, text, language, type, a_set, sort 
                       FROM `%s`.`q_types` 
                       LEFT JOIN `%s`.`q_texts` USING(q_id) 
                       WHERE `q_types`.q_id=%d""" % (self.getDatabase(), self.getDatabase(), edit)
            question = self._queryString(query)
            editQuestion = {'de':'','en':''}
            for q in question :
                editQuestion[q['language']] = q['text']
                editQuestion['type'] = q['type']
                editQuestion['sort'] = q['sort']
                editQuestion['a_set'] = q['a_set']
        # move question
        if up or down :
            qids = self.getQuestionIds(qset)
            if up :
                index = qids.index(up)
                del qids[index]
                qids.insert(index-1,up)
            if down :
                index = qids.index(down)
                del qids[index]
                qids.insert(index+1,down)
            i = 1
            for qid in qids :
                self.moveQuestion(qid,i)
                i+=1
        # get questions
        query = """SELECT q_types.q_id, text, type, a_set, language, enabled 
                   FROM `%s`.`q_types` 
                   LEFT JOIN `%s`.`q_texts` USING(q_id) 
                   WHERE q_set=%s 
                   ORDER BY sort""" % (self.getDatabase(), self.getDatabase(), self._escape(qset))
        questions = self._queryString(query)
        qs = {}
        for question in questions :
            if not qs.has_key(question['q_id']) :
                qs[question['q_id']] = {'type':'', 'de':'-', 'en':'-'}
            qs[question['q_id']][question['language']] = question['text']
            qs[question['q_id']]['type'] = question['type']
            qs[question['q_id']]['a_set'] = question['a_set']
            qs[question['q_id']]['enabled'] = int(question['enabled'])
        # get question ids in right order
        qids = self.getQuestionIds(qset)
        # put everything together in an array containing a dictionary for each question (in the right order)
        qq = []
        for qid in qids :
            qq.append({'id':qid, 'type':qs[qid]['type'], 'de':qs[qid]['de'], 'en':qs[qid]['en'], 'a_set':qs[qid]['a_set'], 'enabled':qs[qid]['enabled']})
        return {'questions':qq,'message':message,'editQuestion':editQuestion}
        
    security.declareProtected(Permissions.access_contents_information, 'getQuestionIds')
    def getQuestionIds(self,set):
        """returns the question ids in the right order"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """SELECT q_id FROM `%s`.`q_types` WHERE q_set=%s ORDER BY sort""" % (self.getDatabase(), self._escape(set))
        qids = self._queryString(query)
        return [x['q_id'] for x in qids]

    security.declareProtected(Permissions.access_contents_information, 'deleteQuestion')
    def deleteQuestion(self,id):
        """deletes the given question"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """DELETE FROM `%s`.`q_types` WHERE q_id=%d""" % (self.getDatabase(), id)
        self._queryString(query)
        query = """DELETE FROM `%s`.`q_texts` WHERE q_id=%d""" % (self.getDatabase(), id)
        self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'enableQuestion')
    def enableQuestion(self,id):
        """enables the given question"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """UPDATE `%s`.`q_types` SET enabled=1 WHERE q_id=%d""" % (self.getDatabase(), id)
        self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'disableQuestion')
    def disableQuestion(self,id):
        """disables the given question"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """UPDATE `%s`.`q_types` SET enabled=0 WHERE q_id=%d""" % (self.getDatabase(), id)
        self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'moveQuestion')
    def moveQuestion(self,id,sort):
        """moves the given question to sort"""
        # check privileges
        if not self.isAdmin() :
            return None
        query = """UPDATE `%s`.`q_types` SET sort=%d WHERE q_id=%d""" % (self.getDatabase(), sort, id)
        self._queryString(query)

    security.declareProtected(Permissions.access_contents_information, 'mailEvaluation')
    def mailEvaluation(self):
        """returns the possible email addresses if lecture is passed
           sends the choosen evaluation to all lecturers if mail_to is passed and returns success message"""
        # check privileges
        if not self.isAdmin() :
            return None
        # return searchLDAP_data(mail_addr=None, mail_domain=None, pers_title=None, pers_sn='xy', pers_givenname=None, pers_room=None, pers_lz=None, pers_ou=None, pers_dept=None)
        req = self.REQUEST
        l_id = getattr(req,'lecture',None)
        mailTo = getattr(req,'mail_to',None)
        if mailTo:
            # send mail
            mailFrom = self.getEmail() or 'evaluation'
            mailTo = [x for x in mailTo if x]
            (name,fileName) = self.getFilename()
            mailSubject = 'Evaluation: %s' % name
            mailText = 'Als Attachment erhalten Sie die Ergebnisse der Evaluation für die Vorlesung:\n%s' % name
            pdf = self._convert2pdf()
            error = False
            for mail in mailTo:
                if not self._mail(mailFrom,mail,mailSubject,mailText,fileName,pdf):
                    error = True
            if not error:
                message = 'Mail erfolgreich versandt.'
            else:
                message = 'Fehler beim Mailversand.'
            return message
        elif l_id:
            # show email form
            semkez = req.selectedSemester
            lecture = self.getLectureWithID(l_id)
            lecturer = lecture['lecturer']
            # fix wrong lecturer format (line breaks)
            lecturer = lecturer.replace('\r',' / ')
            lecturer = lecturer.replace('\n','')
            lecturer = lecturer.replace(';','/')
            lecturers = [x.strip() for x in lecturer.split('/')]
            return self._getEmailForUsers(lecturers)

    security.declarePrivate('_getEmailForUsers')
    def _getEmailForUsers(self,names):
        """returns a dictionary {name:hits} for the given list of names
           hits is a list containing dictionaries {name:,email:} (possible hits)"""
        # check privileges
        if not self.isAdmin() :
            return None
        addresses = {}
        for name in names:
            hits = []
            try:
                firstName = name.split(',')[1]
                firstNames = [x.strip().replace('.','') for x in firstName.split()]
                familyName = name.split(',')[0].strip()
                users = searchLDAP_data(mail_addr=None, mail_domain=None, pers_title=None, pers_sn=familyName, pers_givenname=None, pers_room=None, pers_lz=None, pers_ou=None, pers_dept=None)
                for user in users:
                    if (user['firstName'] == firstName or user['firstName'] in firstNames or user['firstName'][0] in firstNames or user['firstName'].split()[0] in firstNames or user['firstName'].split()[0][0] in firstNames) and user['mail']:  # e.g. L.<>Lorenz / Lorenz<>Lorenz Simon / L. S.<>Lorenz / Peter J.<>Peter Jürg / Samuele G.P.<>Samuele Guido Pio
                        fullName = '%s %s' % (user['firstName'], user['familyName'])
                        hits.append({'name':fullName,'mail':user['mail']})
            except:
                pass
            addresses[name] = hits
        return addresses

    security.declareProtected(Permissions.access_contents_information, 'getEmailAddresses')
    def getEmailAddresses(self):
        """returns the email addresses for all students for the current semester"""
        if self.lectureInscriptionAvailable():
            query = """SELECT DISTINCT `email` FROM `%s`.`inscriptions` WHERE `semkez`=%s""" % (self.getDatabase(), self._escape(self.getCurrentSemester()[0]))
            emails = self._queryString(query)
            return '; '.join([x['email'] for x in emails])
        else:
            emails = {}
            for course in ['BSc','MSc'] :
                ss = getMembersOfGroup(self.getEvaluationGroups()[course])
                for s in ss :
                    student = getLDAP_authEntry(s['PERSID'])[0]
                    try:
                        emails[student['mail'][0]] = 1
                    except:
                        pass
            return '; '.join(emails.keys())

    security.declareProtected(Permissions.access_contents_information, 'getZip')
    def getZip(self):
        """returns the given semester as zip file"""
        # check privileges
        if not self.isAdmin() :
            return None
        req = self.REQUEST
        # get temp file
        (tempHandle, tempName) = mkstemp(suffix=".zip")
        os.close(tempHandle)
        # get zip file
        zip = ZipFile(tempName,'w',ZIP_DEFLATED)
        req.pagename = 'evaluation_view'
        # general questions
        req.type = 'general'
        req.lecture = req.semkez
        for course in ['BSc','MSc']:
            req.course = course
            zip.writestr(self.getFilename()[1],self._convert2pdf())
        # lectures
        req.type = 'lecture'
        for lecture in self.getLectures(req.semkez):
            req.lecture = lecture['l_id']
            req.course = lecture['course']
            zip.writestr(self.getFilename()[1],self._convert2pdf())
        zip.close()
        # read zip
        zipFile = file(tempName, 'rb')
        zipData = zipFile.read()
        zipFile.close()
        os.remove(tempName)
        # send zip
        self.REQUEST.RESPONSE.setHeader('Content-type', 'application/zip')
        self.REQUEST.RESPONSE.setHeader('Content-disposition', 'attachment; filename="evaluation_%s.zip"' % req.semkez)
        return zipData

## authentication methods
    security.declarePublic('getUser') 
    def getUser(self):
        """returns the name of the logged in user (nethz login name)"""
        return getSecurityManager().getUser().getId()

    security.declarePublic('getEmail') 
    def getEmail(self):
        """returns the email address of the logged in user or an empty string if no user is logged in"""
        user = self.getUser()
        try :
            mail = getLDAP_authEntry(get_persid(user))[0]['mail'][0]
        except :
            mail = ''
        return mail
    
    security.declarePublic('getAllRoles')
    def getAllRoles(self):
        """returns all roles for the logged in user (i.e. the highest role and all roles below)"""
        possibleRoles = ['Viewer','Viewer +','Viewer ++','Author','Editor','ChiefEditor','Manager']
        allRoles = []
        roles = self.sec_get_all_roles_for_userid(self.getUser())
        if 'Manager' in roles :
            allRoles = possibleRoles
        elif 'ChiefEditor' in roles :
            allRoles = possibleRoles[:6]
        elif 'Editor' in roles :
            allRoles = possibleRoles[:5]
        elif 'Author' in roles :
            allRoles = possibleRoles[:4]
        elif 'Viewer ++' in roles :
            allRoles = possibleRoles[:3]
        elif 'Viewer +' in roles :
            allRoles = possibleRoles[:2]
        elif 'Viewer' in roles :
            allRoles = possibleRoles[:1]
        return allRoles
    
    security.declarePublic('isLoggedIn')
    def isLoggedIn(self):
        """returns True if user is logged in, otherwise False"""
        if getSecurityManager().getUser().getUserName() == 'Anonymous User' :
            return False
        return True
        
    security.declarePublic('isStudent')
    def isStudent(self):
        """returns the type of student as list ([], [MSc], [BSc] or [MSc,BSc]), can be used also to check if user is a student 
           (this requires no specific Silva role, basic access is managed through Silva access on super folder (normally Anonymous, Authenticated or Viewer))"""
        # if self.getUser() == 'admin' : # DEBUG
        #     return ['BSc']             # DEBUG
        if self.lectureInscriptionAvailable():
            semkez = self.getCurrentSemester()[0]
            query = """SELECT DISTINCT `course` 
                       FROM `%s`.`lectures` 
                       WHERE SUBSTRING(`code`,1,11) IN 
                            (SELECT `code` 
                             FROM `%s`.`inscriptions` 
                             WHERE `u_id`=%s AND `semkez`=%s)
                       AND `semkez`=%s""" % (self.getDatabase(), self.getDatabase(), self._escape(self.getUser()), self._escape(semkez), self._escape(semkez))
            return [x['course'] for x in self._queryString(query)]
        else:
            try :
                groups = getLDAP_authEntry(get_persid(self.getUser()))[0]['ou']
            except :
                groups = []
            student = []
            if self.getEvaluationGroups()['MSc'] in groups :
                student.append('MSc')
            if self.getEvaluationGroups()['BSc'] in groups :
                student.append('BSc')
            return student

    security.declarePublic('isPublicEvaluator')
    def isPublicEvaluator(self):
        """returns True if user can view public evaluation results 
           (Silva Viewer +)"""
        # if self.getUser() == 'admin' : # DEBUG
        #     return False               # DEBUG
        if 'Viewer +' in self.getAllRoles() :
            return True
        return False

    security.declarePublic('isEvaluator')
    def isEvaluator(self):
        """returns True if user can view all evaluation results (i.e. also evaluations which are not public) 
           (Silva Viewer ++)"""
        # if self.getUser() == 'admin' : # DEBUG
        #     return False               # DEBUG
        if 'Viewer ++' in self.getAllRoles() :
            return True
        return False

    security.declarePublic('isTextEvaluator')
    def isTextEvaluator(self):
        """returns True if user can write summaries of text answers 
           (Silva Author)"""
        # if self.getUser() == 'admin' : # DEBUG
        #     return False               # DEBUG
        if 'Author' in self.getAllRoles() :
            return True
        return False
    
    security.declarePublic('isAdmin')
    def isAdmin(self):
        """returns True if user is administrator (i.e. he can set up new evaluations and change existing ones) 
           (Silva Editor)"""
        # if self.getUser() == 'admin' : # DEBUG
        #     return False               # DEBUG
        if 'Editor' in self.getAllRoles() :
            return True
        return False

    security.declareProtected(Permissions.access_contents_information,'lectureInscriptionAvailable')
    def lectureInscriptionAvailable(self,semkez=None):
        """returns True if lecture inscriptions are available for the given semester
           if no semkez the current semkez is checked
           (i.e. a text file has been uploaded which contains the lecture inscriptions for each student)"""
        if not semkez:
            semkez = self.getCurrentSemester()[0]
        query = """SELECT COUNT(*) AS `count` FROM `%s`.`inscriptions` WHERE `semkez`=%s""" % (self.getDatabase(), self._escape(semkez))
        count = int(self._queryString(query)[0]['count'])
        return count and True or False

    security.declareProtected(Permissions.access_contents_information,'getInscriptionsForStudent')
    def getInscriptionsForStudent(self,student=None,semkez=None):
        """returns the lectures for which the given student is inscripted in the given semkez
           if no student is given it returns the inscriptions for the current logged in user
           if no semkez is given it uses the current semkez"""
        if not student or not self.isAdmin():  # only return inscriptions for other students to admins
            student = self.getUser()
        if not semkez:
            semkez = self.getCurrentSemester()[0]
        query = """SELECT * 
                   FROM `%s`.`lectures` 
                   WHERE SUBSTRING(`code`,1,11) IN 
                        (SELECT `code` 
                         FROM `%s`.`inscriptions` 
                         WHERE `u_id`=%s AND `semkez`=%s)
                   AND `semkez`=%s
                   ORDER BY `name`""" % (self.getDatabase(), self.getDatabase(), self._escape(student), self._escape(semkez), self._escape(semkez))
        return self._queryString(query)


## helper methods    
    security.declarePublic('getURL') 
    def getURL(self, args={}):
        """returns the complete url with GET arguments
           adds arguments from dictionary args if passed
           cannot be used if get/request vars with equal names (e.g. "ids:list")
        """
        url = self.REQUEST.URL0
        queryString = self.REQUEST.QUERY_STRING
        # get GET args
        getArgs = {}
        if queryString :
            for arg in queryString.split('&') :
                (key,value) = [unquote(x) for x in arg.split('=')]
                getArgs[key] = value
        # add passed args
        getArgs.update(args)
        # make url
        queryString = urlencode(getArgs)
        if queryString :
            url = '%s?%s' % (url, queryString)
        return url
    
    security.declarePublic('getContext')
    def getContext(self):
        """returns the context (main, questions, evaluation, abstracts, admin)"""
        url = self.REQUEST.URL0
        page = url.split('/')[-1]
        if page.count('_') :
            return page.split('_')[0]
        else :
            return 'main'

    security.declarePublic('getHomeLink') 
    def getHomeLink(self):
        """returns the home link for the appropriate context (questions_index, evaluation_index, abstracts_index, admin_index)"""
        context = self.getContext()
        if context in ['questions', 'evaluation', 'abstracts', 'admin'] :
            return getattr(self, context+'_index').absolute_url()
        else :
            return self.index.absolute_url()

    security.declarePublic('getAbsoluteLink') 
    def getAbsoluteLink(self):
        """returns the absolute link to the evaluation tool"""
        return self.absolute_url()

    security.declareProtected(Permissions.view_management_screens, 'getLecturesEditors')
    def getLecturesEditors(self):
        """returns all available courses"""
        editors = [x.getId() for x in self.get_container().objectValues('DBEditor')]
        folders = self.get_container().objectValues('Silva Folder')
        for folder in folders:
            editors += ['%s/%s' % (folder.getId(),x.getId()) for x in folder.objectValues('DBEditor')]
        return  editors

    security.declarePublic('imageLink') 
    def imageLink(self, image):
        """returns an absolute url for the given image"""
        return '%s/%s' % (self.absolute_url(), image)
        
    security.declarePublic('htmlEncode') 
    def htmlEncode(self, string):
        """
        returns the given string html encoded.
        """
        return newline_to_br(html_quote(string))

    security.declarePublic('urlquote') 
    def urlquote(self, string):
        """
        returns the given string quoted to be used in an url.
        """
        return quote(string)
    
    security.declarePublic('getLanguage') 
    def getLanguage(self):
        """returns the language (from the session)"""
        session = self.REQUEST.SESSION
        if session.has_key('language') and self.getContext() not in ['evaluation', 'abstracts', 'admin'] :
            return session['language']
        else :
            # set language to german for evaluation/admin sites (because they are not localized)
            return 'de'
    
    security.declarePublic('setLanguage') 
    def setLanguage(self, language=None):
        """sets the language (if no language passed it takes language from request if there is any)"""
        request = self.REQUEST
        session = request.SESSION
        if language :
            session['language'] = language
        else :
            # get language from request
            if request.has_key('language') :
                session['language'] = request['language']

    security.declarePublic('getSelectedSemester') 
    def getSelectedSemester(self):
        """returns the semkez of the selected semester and saves the selected semester if passed by request
           (used in evaluation_index and abstracts_index)"""
        request = self.REQUEST
        session = request.SESSION
        if request.has_key('selectedSemester') :
            session['selectedSemester'] = request['selectedSemester']
        if session.has_key('selectedSemester') :
            return session['selectedSemester']
        else :
            return self.getCurrentSemester()[0]

    security.declarePublic('getCurrentSemester')   # has to be public so that semester can be shown in education box_feeder
    def getCurrentSemester(self):
        """Returns the current semester as tuple (semkez,semlang)"""
        ss = self.getAllSemesters()
        if len(ss) > 0 :
            return ss[-1]
        else :
            return (None,None)
    
    security.declareProtected(Permissions.access_contents_information, 'getAllSemesters')
    def getAllSemesters(self):
        """Returns all semesters as array of tuples (semkez,semlang)"""
        query = "SELECT semkez, semlang FROM `%s`.`lectures` GROUP BY semkez" % (self.getDatabase())
        return [(row['semkez'],row['semlang']) for row in self._queryString(query)]
    
    security.declareProtected(Permissions.access_contents_information, 'getSemlangForSemkez')
    def getSemlangForSemkez(self, semkez):
        """returns semlang for the given semkez (taken from the database)"""
        query = "SELECT semlang FROM `%s`.`lectures` WHERE semkez=%s" % (self.getDatabase(), self._escape(semkez))
        result = self._queryString(query)
        if result :
            return result[0]['semlang']
        else :
            return ''

    security.declarePublic('getPDFLink')
    def getPDFLink(self):
        """returns the link to get the current page as pdf"""
        req = self.REQUEST
        page = req.URL0.split('/')[-1]
        link = '%s/getPDF?pagename=%s' % (self.absolute_url(), page)
        queryString = self.getQueryString()
        if queryString :
            link += '&%s' % queryString
        return link

    security.declarePublic('getMailLink')
    def getMailLink(self):
        """returns the link to mail the page as pdf"""
        req = self.REQUEST
        page = req.URL0.split('/')[-1]
        link = '%s/mailPage?pagename=%s' % (self.absolute_url(), page)
        queryString = self.getQueryString()
        if queryString :
            link += '&%s' % queryString
        return link

    security.declarePublic('getQueryString')
    def getQueryString(self, exclude=[]):
        """returns the query string for the current get/post request
           if a list is passed for exclude keys contained in that list are ignored"""
        req = self.REQUEST
        # get get/post values
        vars = {}
        listVars = []
        for var in req.form.keys() :
            if var != '-C' and var not in exclude :  # zope bug returning -C if no GET query string
                if isinstance(req.form[var],list) :
                    for item in req.form[var] :
                        key = '%s:list' % var
                        listVars.append({key:item})  # we have to use a list of dictionaries because the keys are the same
                else :
                    vars[var] = req.form[var]
        queryString = urlencode(vars)
        for item in listVars :
            queryString += '&%s' % urlencode(item)
        return queryString

    security.declarePublic('getQueryString')
    def getRequestAsHidden(self, exclude=[]):
        """returns hidden fields for the REQUEST query string
           if a list is passed for exclude keys contained in that list are ignored"""
        req = self.REQUEST
        hiddenTemplate = """<input type="hidden" name="%s" value="%s">"""
        # get get/post values
        hidden = ''
        for var in req.form.keys() :
            if var != '-C' and var not in exclude :  # zope bug returning -C if no GET query string
                if isinstance(req.form[var],list) :
                    for item in req.form[var] :
                        key = '%s:list' % var
                        hidden += hiddenTemplate % (key, item)
                else :
                    hidden += hiddenTemplate % (var, req.form[var])
        return hidden

    security.declareProtected(Permissions.access_contents_information, 'getPDF')
    def getPDF(self):
        """returns a pdf of the page"""
        # send pdf
        self.REQUEST.RESPONSE.setHeader('Content-type', 'application/pdf')
        self.REQUEST.RESPONSE.setHeader('Content-disposition', 'inline; filename="%s"' % self.getFilename()[1])
        return self._convert2pdf()

    security.declarePublic('excludeFromPDF')
    def excludeFromPDF(self):
        """returns False if we're in a PDF method, otherwise True
           used to hide elements from pdf generation"""
        return self.REQUEST.URL0.split('/')[-1] not in ['getPDF','mailPage', 'admin_mailevaluation', 'getZip']

    security.declareProtected(Permissions.access_contents_information, 'mailAsPDF')
    def mailAsPDF(self):
        """mails the selected page as PDF to the recipient specified in the form
           returns a tuple (message,backLink,lectureName)"""
        req = self.REQUEST
        message = ''
        mailTo = getattr(self.REQUEST,'mail_to',None)
        (name,fileName) = self.getFilename()
        if mailTo:
            mailFrom = req.mail_from
            if not mailFrom:
                mailFrom = 'evaluation'
            mailSubject = req.mail_subject
            mailText = req.mail_text
            pdf = self._convert2pdf()
            # mail page
            if self._mail(mailFrom,mailTo,mailSubject,mailText,fileName,pdf):
                message = 'Mail erfolgreich versandt.'
            else:
                message = 'Fehler beim Mailversand.'
        # get back link
        queryString = self.getQueryString(['mail_from','mail_to','mail_subject','mail_text','pagename'])
        backLink = '%s/%s?%s' % (self.absolute_url(),req.pagename,queryString)
        return (message,backLink,name)

    security.declareProtected(Permissions.access_contents_information, 'getFilename')
    def getFilename(self):
        """returns a tuple (name,filename) to be used for PDF creation and mailing"""
        # get lecture (for subject and filename)
        req = self.REQUEST
        l_id = getattr(req,'lecture',None)
        type = getattr(req,'type',None)
        course = getattr(req,'course',None)
        name = ''
        fileName = 'evaluation.pdf'
        if l_id and type and course:
            lecture = self.getLectureWithID(l_id,type)
            if type == 'general':
                name = '%s (%s %s)' % (lecture['name'],lecture['semlang'],course)
                fileName = 'evaluation_%s_allgemeine_fragen_%s.pdf' % (lecture['semkez'], course)
            else:
                name = '%s: %s (%s %s)' % (lecture['code'],lecture['name'],lecture['semlang'],lecture['course'])
                fileName = 'evaluation_%s_%s.pdf' % (lecture['semkez'],lecture['code'])
        return (name,fileName)

    security.declarePrivate('_mail')
    def _mail(self,mailFrom,mailTo,mailSubject,mailText,fileName,pdf):
        """sends a mail using the given arguments
           returns True on success, False otherwise"""
        try:
            message = StringIO.StringIO()
            writer = MimeWriter.MimeWriter(message)
            writer.addheader('MIME-Version', '1.0')
            writer.addheader('Subject', mailSubject)
            writer.addheader('To', mailTo)
            writer.startmultipartbody('mixed')
            # start off with a text/plain part
            part = writer.nextpart()
            part.addheader('Content-Transfer-Encoding', '8bit')
            body = part.startbody('text/plain; charset="UTF-8"')
            body.write('%s\n\n' % mailText)
            # now add an image part
            part = writer.nextpart()
            part.addheader('Content-Transfer-Encoding', 'base64')
            body = part.startbody('application/pdf; name=%s' % fileName)
            body.write(base64.encodestring(pdf))
            # finish off
            writer.lastpart()
            # send the mail
            smtp = smtplib.SMTP(smtp_host)
            smtp.sendmail(mailFrom, mailTo, message.getvalue())
            smtp.quit()
            return True
        except:
            return False

    security.declarePrivate('_convert2pdf')
    def _convert2pdf(self):
        """converts the html page to pdf taking arguments from request"""
        pageName = self.REQUEST.pagename
        page = getattr(self,pageName)
        # self.REQUEST.RESPONSE.setHeader('Content-type', 'text/html;charset=UTF-8')
        content = page()
        # fix image links in page (-> to non secure image location)
        if getattr(self,'images_evaluation',None):
            path = self.images_evaluation.absolute_url()
            for image in _images:
                imgOld = 'src="%s"' % self.imageLink(image.replace('.','_'))
                imgNew = 'src="%s/%s"' % (path,image)
                content = content.replace(imgOld,imgNew)
        # get temp file
        (tempHandle, tempName) = mkstemp(suffix=".html")
        os.close(tempHandle)
        # write page to temp file
        f = open(tempName, 'r+w')
        f.write(content.encode('iso-8859-15', 'replace'))
        f.close()
        # get pdf
        # (stout, stin) = popen2('htmldoc  --header . --fontsize 10 --bodyfont helvetica --webpage -t pdf --quiet --jpeg "%s"' % tempName)
        # (stout, stin) = popen2('htmldoc  --header . --footer hd/ --fontsize 10 --headfootsize 8 --bodyfont helvetica --headfootfont helvetica --size a4 --left 1cm --right 1cm --top 1cm --bottom 1cm --webpage -t pdf --quiet --jpeg "%s"' % tempName)
        (stout, stin) = popen2('htmldoc  --header . --fontsize 10 --headfootsize 8 --bodyfont helvetica --headfootfont helvetica --size a4 --left 1cm --right 1cm --top 1cm --bottom 1cm --no-links --webpage -t pdf --quiet --jpeg "%s"' % tempName)
        pdf = stout.read()
        stout.close()
        stin.close()
        os.remove(tempName)
        # send pdf
        return pdf

    security.declarePublic('getDateTime')
    def getDateTime(self):
        """returns the date and time in format DD-MM-YYYY HH:MM:SS"""
        return strftime('%d.%m.%Y %H:%M:%S')

    security.declarePublic('getSemesterOfStudent')
    def getSemesterOfStudent(self, groups=None, student=None, course=None):
        """return the semester parsing the given groups (or 0 if there is no semester)
           if no groups are given it takes the groups of the given student (nethz)
           if no groups and no student are given it takes the groups of the currently logged in user
           if course is given the semester for this course is returned, otherwise MSc is first checked and then BSc and the first semester found is returned"""
        # if not groups and not student and self.getUser() == 'admin' : # DEBUG
        #     return 3                                                  # DEBUG
        if not groups :
            if not student :
                student = self.getUser()
            try :
                groups = getLDAP_authEntry(get_persid(student))[0]['ou']
            except :
                groups = []
        # get semester
        if course :
            courses = [course]
        else :
            courses = ['MSc','BSc']
        semester = 0
        for group in groups :
            for course in courses :
                exp = escape(self._semesterGroups[course]).replace('\%','(\d*)')
                m = match(exp,group)
                if m :
                    semester = int(m.groups()[0])
                    break
        return int(semester)

    security.declareProtected(Permissions.view_management_screens, 'getAvailableCourses')
    def getAvailableCourses(self):
        """returns all available courses"""
        try:
            cache = get_vvz_cache(self)
            semkez = self.getAvailableVVZSemesters()[-1][0]
            semester = cache.get_semester(semkez)
            courses = [x.title for x in semester.getCourses().objectValues()]
            courses.sort()
            return courses
        except Exception, e:
            return []


## variable getters
    security.declareProtected(Permissions.access_contents_information,'footer')
    def footer(self):
        """Returns the footer to be displayed at the bottom of every page."""
        return self._footer

    security.declareProtected(Permissions.access_contents_information,'version')
    def version(self):
        """Returns the version number."""
        version = file(os.path.join(package_home(globals()), 'version.txt')).read()
        version = version.split()[1]
        return version

    security.declareProtected(Permissions.access_contents_information,'contactLink')
    def contactLink(self):
        """returns the contact link (used on entry page)"""
        return self._contactLink

    security.declareProtected(Permissions.access_contents_information, 'getBscCourse')
    def getBscCourse(self):
        """returns the name of the bachelor course"""
        return self._bscCourse

    security.declareProtected(Permissions.access_contents_information, 'getMscCourse')
    def getMscCourse(self):
        """returns the name of the bachelor course"""
        return self._mscCourse

    security.declareProtected(Permissions.access_contents_information, 'getEvaluationGroups')
    def getEvaluationGroups(self):
        """returns the evaluation groups (ldap-groups for allowing access to the questionnaire)"""
        return self._evaluationGroups

    security.declareProtected(Permissions.access_contents_information, 'getSemesterGroups')
    def getSemesterGroups(self):
        """returns the semester groups (used to parse the ldap-groups for semester of student)"""
        return self._semesterGroups

    security.declareProtected(Permissions.access_contents_information,'getLecturesEditorName')
    def getLecturesEditorName(self):
        """returns the saved lectures editor name (e.g. editor or folder/editor)"""
        return self._lecturesEditor


## MySQL methods
    security.declareProtected(Permissions.view_management_screens,'getDbConnections')
    def getDbConnections(self):
        """Returns all available db connections"""
        ids = SQLConnectionIDs(self)
        return [x[1] for x in ids]
        
    security.declareProtected(Permissions.view_management_screens,'getDbConnectionId')
    def getDbConnectionId(self):
        """Returns the current dbConnectionId"""
        return self._dbConnectionId

    security.declareProtected(Permissions.view_management_screens,'setDbConnectionId')
    def setDbConnectionId(self, dbConnectionId):
        """Sets the dbConnectionId and tries connecting to the db
        returns True if successfully connected, otherwise False"""
        self._dbConnectionId = dbConnectionId
        self._dbConnection = getattr(self, self._dbConnectionId, None)
        if self._dbConnection :
            return True
        else :
            self._dbConnectionId = ''
            return False

    security.declareProtected(Permissions.access_contents_information, 'databaseIsConnected') 
    def databaseIsConnected(self):
        """Returns true if database is connected."""
        # refresh connection in case it has changed
        self._dbConnection = getattr(self, self._dbConnectionId, None)
        if not self._dbConnection :
            return False
        return True

    security.declareProtected(Permissions.view_management_screens, 'getDatabases') 
    def getDatabases(self):
        """Returns all databases."""
        if not self.databaseIsConnected() :
            return []
        result = self._queryString("SHOW DATABASES")
        databases = [row[0] for row in result]
        return databases
        
    security.declareProtected(Permissions.view_management_screens, 'getDatabase') 
    def getDatabase(self):
        """Returns the database to be used for the evaluation."""
        return self._database
        
    security.declarePrivate('_queryString') 
    def _queryString(self, query, error=True):
        """Performs the given string as MySQL query.
           If error is False, MySQL errors are not saved to _error (i.e. getLastErrorMessage() will still return the error from the last MySQL query)."""
        # refresh connection in case it has changed
        self._dbConnection = getattr(self, self._dbConnectionId, None)
        # set encoding to utf8
        proc_id = "utf8_proc"
        sql_method = SQL(proc_id, '', self._dbConnectionId, {}, "SET NAMES 'utf8'")
        setattr(sql_method, self._dbConnectionId, self._dbConnection)
        sql_method(REQUEST={})
        # perform query
        proc_id = "query_proc"
        sql_method = SQL(proc_id, '', self._dbConnectionId, {}, query)
        # disable result limit
        sql_method.max_rows = sql_method.max_rows_ = 0
        setattr(sql_method, self._dbConnectionId, self._dbConnection)
        try :
            result = sql_method(REQUEST={})
            if error :
                self._error = None
        except Exception,e :
            result = []
            if error :
                self._error = str(e)+' query was: '+query
            raise e # DEBUG
        return result

    security.declareProtected(Permissions.access_contents_information, 'getLastErrorMessage') 
    def getLastErrorMessage(self):
        """Returns the error message from the last query or None if there was no error."""
        return self._error

    security.declarePrivate('_escape') 
    def _escape(self, object):
        """Escapes a string/int or array/dictionary for use in MySQL.
           object can be a string/int or an array. If it is an array,
           all elements in it are escaped. If a value is None, it is replaced by NULL.
           Elements are also quoted (except NULL)."""
        if object == None :
            object = 'NULL'
        elif isinstance(object,str) or isinstance(object,int) or isinstance(object,long) :
            object = escape_string(str(object))
            object = "'%s'" % object
        else :
            for i in xrange(len(object)) :
                if object[i] == None :
                    object[i] = 'NULL'
                else :
                    object[i] = "'%s'" % escape_string(str(object[i]))
        return object
    
    security.declareProtected(Permissions.access_contents_information, 'stringFromBlob')
    def stringFromBlob(self, blob):
        """
        returns a string from the given blob (works also if a string or int is passed)"""
        if isinstance(blob,array) :
            return blob.tostring()
        else :
            return str(blob)
    
    ## DEBUG
    # security.declarePrivate('_log')
    # def _log(self, key, value):
    #     """log to mysql db"""
    #     query = """INSERT INTO `%s`.`log`
    #                (`key`, `value`)
    #                VALUES
    #                (%s, %s)""" % (self.getDatabase(), self._escape(key), self._escape(value))
    #     self._queryString(query)


## add object methods
InitializeClass(Evaluation)

manage_addEvaluationForm = PageTemplateFile(
        'www/EvaluationAdd', globals(),
        __name__ = 'manage_addEvaluationForm')

def manage_addEvaluation(dispatcher, id, title, dbConnectionId='', REQUEST=None):
    """
    Create a new Evaluation tool and add it to myself.
    """
    p = Evaluation(id, unicode(title, 'UTF-8'), unicode(dbConnectionId, 'UTF-8'))
    dispatcher.Destination()._setObject(id, p)
    add_and_edit(dispatcher, id, REQUEST)
    return ''
